@@ -1,22 +1,20 @@
 CREATE FUNCTION @extschema@.create_parent(
     p_parent_table text
     , p_control text
-    , p_type text
     , p_interval text
-    , p_constraint_cols text[] DEFAULT NULL 
+    , p_type text DEFAULT 'range'
+    , p_epoch text DEFAULT 'none'
     , p_premake int DEFAULT 4
-    , p_automatic_maintenance text DEFAULT 'on' 
     , p_start_partition text DEFAULT NULL
-    , p_inherit_fk boolean DEFAULT true
-    , p_epoch text DEFAULT 'none' 
-    , p_upsert text DEFAULT ''
-    , p_publications text[] DEFAULT NULL
-    , p_trigger_return_null boolean DEFAULT true
+    , p_default_table boolean DEFAULT true
+    , p_automatic_maintenance text DEFAULT 'on'
+    , p_constraint_cols text[] DEFAULT NULL
     , p_template_table text DEFAULT NULL
     , p_jobmon boolean DEFAULT true
-    , p_date_trunc_interval text DEFAULT NULL)
-RETURNS boolean 
-    LANGUAGE plpgsql 
+    , p_date_trunc_interval text DEFAULT NULL
+)
+    RETURNS boolean
+    LANGUAGE plpgsql
     AS $$
 DECLARE
 
@@ -37,12 +35,11 @@ v_higher_parent_epoch           text;
 v_higher_parent_schema          text := split_part(p_parent_table, '.', 1);
 v_higher_parent_table           text := split_part(p_parent_table, '.', 2);
 v_id_interval                   bigint;
-v_inherit_privileges            boolean := false;
+v_inherit_privileges            boolean := false; -- This is false by default so initial partition set creation doesn't require superuser.
 v_job_id                        bigint;
 v_jobmon_schema                 text;
 v_last_partition_created        boolean;
 v_max                           bigint;
-v_native_sub_control            text;
 v_notnull                       boolean;
 v_new_search_path               text;
 v_old_search_path               text;
@@ -51,26 +48,22 @@ v_parent_partition_id           bigint;
 v_parent_partition_timestamp    timestamptz;
 v_parent_schema                 text;
 v_parent_tablename              text;
-v_parent_tablespace             text;
 v_part_col                      text;
 v_part_type                     text;
 v_partition_time                timestamptz;
 v_partition_time_array          timestamptz[];
 v_partition_id_array            bigint[];
 v_partstrat                     char;
-v_publication_exists            text;
 v_row                           record;
 v_sql                           text;
 v_start_time                    timestamptz;
 v_starting_partition_id         bigint;
 v_step_id                       bigint;
 v_step_overflow_id              bigint;
-v_sub_parent                    text;
 v_success                       boolean := false;
 v_template_schema               text;
 v_template_tablename            text;
 v_time_interval                 interval;
-v_top_datetime_string           text;
 v_top_parent_schema             text := split_part(p_parent_table, '.', 1);
 v_top_parent_table              text := split_part(p_parent_table, '.', 2);
 v_unlogged                      char;
@@ -86,34 +79,36 @@ ELSIF array_length(string_to_array(p_parent_table, '.'), 1) > 2 THEN
     RAISE EXCEPTION 'pg_partman does not support objects with periods in their names';
 END IF;
 
-IF p_upsert <> '' THEN
-    IF current_setting('server_version_num')::int < 90500 THEN
-        RAISE EXCEPTION 'INSERT ... ON CONFLICT (UPSERT) feature is only supported in PostgreSQL 9.5 and later';
-    END IF;
-    IF p_type = 'native' AND current_setting('server_version_num')::int >= 110000 THEN
-        RAISE EXCEPTION 'The pg_partman upsert feature is not supported with native partitioning in PG11+. Use the built-in support for INSERT ON CONFLICT with native partitioning instead.';
-    END IF;
+IF p_interval = 'yearly'
+    OR p_interval = 'quarterly'
+    OR p_interval = 'monthly'
+    OR p_interval  = 'weekly'
+    OR p_interval = 'daily'
+    OR p_interval = 'hourly'
+    OR p_interval = 'half-hour'
+    OR p_interval = 'quarter-hour'
+THEN
+    RAISE EXCEPTION 'Special partition interval values from old pg_partman versions (%) are no longer supported. Please use a supported interval time value from core PostgreSQL (https://www.postgresql.org/docs/current/datatype-datetime.html#DATATYPE-INTERVAL-INPUT)', p_interval;
 END IF;
 
-SELECT n.nspname, c.relname, t.spcname, c.relpersistence
-INTO v_parent_schema, v_parent_tablename, v_parent_tablespace, v_unlogged
+SELECT n.nspname, c.relname, c.relpersistence
+INTO v_parent_schema, v_parent_tablename, v_unlogged
 FROM pg_catalog.pg_class c
 JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-LEFT OUTER JOIN pg_catalog.pg_tablespace t ON c.reltablespace = t.oid
 WHERE n.nspname = split_part(p_parent_table, '.', 1)::name
 AND c.relname = split_part(p_parent_table, '.', 2)::name;
     IF v_parent_tablename IS NULL THEN
         RAISE EXCEPTION 'Unable to find given parent table in system catalogs. Please create parent table first: %', p_parent_table;
     END IF;
-    
-SELECT attnotnull INTO v_notnull 
+
+SELECT attnotnull INTO v_notnull
 FROM pg_catalog.pg_attribute a
 JOIN pg_catalog.pg_class c ON a.attrelid = c.oid
 JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
 WHERE c.relname = v_parent_tablename::name
 AND n.nspname = v_parent_schema::name
 AND a.attname = p_control::name;
-    IF p_type <> 'native' AND (v_notnull = false OR v_notnull IS NULL) THEN
+    IF (v_notnull = false OR v_notnull IS NULL) THEN
         RAISE EXCEPTION 'Control column given (%) for parent table (%) does not exist or must be set to NOT NULL', p_control, p_parent_table;
     END IF;
 
@@ -125,7 +120,7 @@ IF v_control_type IS NULL THEN
 END IF;
 
 IF (p_epoch <> 'none' AND v_control_type <> 'id') THEN
-    RAISE EXCEPTION 'p_epoch can only be used with an integer based control column and does not work for native partitioning';
+    RAISE EXCEPTION 'p_epoch can only be used with an integer based control column';
 END IF;
 
 
@@ -133,114 +128,73 @@ IF NOT @extschema@.check_partition_type(p_type) THEN
     RAISE EXCEPTION '% is not a valid partitioning type for pg_partman', p_type;
 END IF;
 
-IF p_type = 'native' THEN
+IF current_setting('server_version_num')::int < 140000 THEN
+    RAISE EXCEPTION 'pg_partman requires PostgreSQL 14 or greater';
+END IF;
+-- Check if given parent table has been already set up as a partitioned table and is ranged
+SELECT p.partstrat, partattrs INTO v_partstrat, v_partattrs
+FROM pg_catalog.pg_partitioned_table p
+JOIN pg_catalog.pg_class c ON p.partrelid = c.oid
+JOIN pg_namespace n ON c.relnamespace = n.oid
+WHERE n.nspname = v_parent_schema::name
+AND c.relname = v_parent_tablename::name;
 
-    IF current_setting('server_version_num')::int < 100000 THEN
-        RAISE EXCEPTION 'Native partitioning only available in PostgreSQL versions 10.0+';
-    END IF;
-    -- Check if given parent table has been already set up as a partitioned table and is ranged
-    SELECT p.partstrat, partattrs INTO v_partstrat, v_partattrs
-    FROM pg_catalog.pg_partitioned_table p
-    JOIN pg_catalog.pg_class c ON p.partrelid = c.oid
-    JOIN pg_namespace n ON c.relnamespace = n.oid
-    WHERE n.nspname = v_parent_schema::name 
-    AND c.relname = v_parent_tablename::name;
-
-    IF v_partstrat <> 'r' OR v_partstrat IS NULL THEN
-        RAISE EXCEPTION 'When using native partitioning, you must have created the given parent table as ranged (not list) partitioned already. Ex: CREATE TABLE ... PARTITION BY RANGE ...)';
-    END IF;
-
-    IF array_length(v_partattrs, 1) > 1 THEN
-        RAISE NOTICE 'pg_partman only supports single column native partitioning at this time. Found % columns in given parent definition.', array_length(v_partattrs, 1);
-    END IF;
-
-    SELECT a.attname, t.typname
-    INTO v_part_col, v_part_type
-    FROM pg_attribute a
-    JOIN pg_class c ON a.attrelid = c.oid
-    JOIN pg_namespace n ON c.relnamespace = n.oid
-    JOIN pg_type t ON a.atttypid = t.oid
-    WHERE n.nspname = v_parent_schema::name
-    AND c.relname = v_parent_tablename::name
-    AND attnum IN (SELECT unnest(partattrs) FROM pg_partitioned_table p WHERE a.attrelid = p.partrelid);
-
-    IF p_control <> v_part_col OR v_control_exact_type <> v_part_type THEN
-        RAISE EXCEPTION 'Control column and type given in arguments (%, %) does not match the control column and type of the given native partition set (%, %)', p_control, v_control_exact_type, v_part_col, v_part_type;
-    END IF;
-
-    -- Check that control column is a usable type for pg_partman.
-    IF v_control_type NOT IN ('time', 'id') THEN
-        RAISE EXCEPTION 'Only date/time or integer types are allowed for the control column with native partitioning.';
-    END IF;
-
-    -- Table to handle properties not natively inherited yet (indexes, fks, etc)
-    IF p_template_table IS NULL THEN
-        v_template_schema := '@extschema@';
-        v_template_tablename := @extschema@.check_name_length('template_'||v_parent_schema||'_'||v_parent_tablename);
-        EXECUTE format('CREATE TABLE IF NOT EXISTS %I.%I (LIKE %I.%I)', '@extschema@', v_template_tablename, v_parent_schema, v_parent_tablename);
-
-        SELECT pg_get_userbyid(c.relowner) INTO v_parent_owner 
-        FROM pg_catalog.pg_class c
-        JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE n.nspname = v_parent_schema::name 
-        AND c.relname = v_parent_tablename::name;
-
-        EXECUTE format('ALTER TABLE %I.%I OWNER TO %I'
-                , '@extschema@' 
-                , v_template_tablename 
-                , v_parent_owner);
-    ELSE
-        SELECT n.nspname, c.relname INTO v_template_schema, v_template_tablename
-        FROM pg_catalog.pg_class c
-        JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE n.nspname = split_part(p_template_table, '.', 1)::name
-        AND c.relname = split_part(p_template_table, '.', 2)::name;
-            IF v_template_tablename IS NULL THEN
-                RAISE EXCEPTION 'Unable to find given template table in system catalogs (%). Please create template table first or leave parameter NULL to have a default one created for you.', p_parent_table;
-            END IF;
-    END IF;
-
-ELSE -- if not native 
-
-    IF current_setting('server_version_num')::int >= 100000 THEN
-        SELECT p.partstrat INTO v_partstrat
-        FROM pg_catalog.pg_partitioned_table p
-        JOIN pg_catalog.pg_class c ON p.partrelid = c.oid
-        JOIN pg_namespace n ON c.relnamespace = n.oid
-        WHERE n.nspname = v_parent_schema::name 
-        AND c.relname = v_parent_tablename::name;
-    END IF;
-
-    IF v_partstrat IS NOT NULL THEN
-        RAISE EXCEPTION 'Given parent table has been set up with native partitioning therefore cannot be used with pg_partman''s other partitioning types. Either recreate table non-native or set the type argument to ''native''';
-    END IF;
-
-END IF; -- end if "native" check
-
-
-IF p_publications IS NOT NULL THEN
-    IF current_setting('server_version_num')::int < 100000 THEN
-        RAISE EXCEPTION 'p_publications argument not null but CREATE PUBLICATION is only available in PostgreSQL versions 10.0+';
-    END IF;
-    IF p_publications = '{}' THEN
-        RAISE EXCEPTION 'p_publications cannot be an empty set';
-    END IF;
-    FOR v_row IN 
-        SELECT unnest(p_publications) AS pubname
-    LOOP
-        SELECT pubname INTO v_publication_exists FROM pg_catalog.pg_publication where pubname = v_row.pubname::name;
-        IF v_publication_exists IS NULL THEN
-            RAISE EXCEPTION 'Given publication name (%) does not exist in system catalog. Ensure it is created first.', v_row.pubname;
-        END IF;
-    END LOOP;
+IF v_partstrat <> 'r' OR v_partstrat IS NULL THEN
+    RAISE EXCEPTION 'You must have created the given parent table as ranged (not list) partitioned already. Ex: CREATE TABLE ... PARTITION BY RANGE ...)';
 END IF;
 
--- Only inherit parent ownership/privileges on non-native sets by default
--- This is false by default so initial partition set creation doesn't require superuser.
-IF p_type = 'native' THEN
-    v_inherit_privileges = false;
+IF array_length(v_partattrs, 1) > 1 THEN
+    RAISE NOTICE 'pg_partman only supports single column partitioning at this time. Found % columns in given parent definition.', array_length(v_partattrs, 1);
+END IF;
+
+SELECT a.attname, t.typname
+INTO v_part_col, v_part_type
+FROM pg_attribute a
+JOIN pg_class c ON a.attrelid = c.oid
+JOIN pg_namespace n ON c.relnamespace = n.oid
+JOIN pg_type t ON a.atttypid = t.oid
+WHERE n.nspname = v_parent_schema::name
+AND c.relname = v_parent_tablename::name
+AND attnum IN (SELECT unnest(partattrs) FROM pg_partitioned_table p WHERE a.attrelid = p.partrelid);
+
+IF p_control <> v_part_col OR v_control_exact_type <> v_part_type THEN
+    RAISE EXCEPTION 'Control column and type given in arguments (%, %) does not match the control column and type of the given partition set (%, %)', p_control, v_control_exact_type, v_part_col, v_part_type;
+END IF;
+
+-- Check that control column is a usable type for pg_partman.
+IF v_control_type NOT IN ('time', 'id') THEN
+    RAISE EXCEPTION 'Only date/time or integer types are allowed for the control column.';
+END IF;
+
+-- Table to handle properties not managed by core PostgreSQL yet
+IF p_template_table IS NULL THEN
+    v_template_schema := '@extschema@';
+    v_template_tablename := @extschema@.check_name_length('template_'||v_parent_schema||'_'||v_parent_tablename);
+    EXECUTE format('CREATE TABLE IF NOT EXISTS %I.%I (LIKE %I.%I)', v_template_schema, v_template_tablename, v_parent_schema, v_parent_tablename);
+
+    SELECT pg_get_userbyid(c.relowner) INTO v_parent_owner
+    FROM pg_catalog.pg_class c
+    JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
+    WHERE n.nspname = v_parent_schema::name
+    AND c.relname = v_parent_tablename::name;
+
+    EXECUTE format('ALTER TABLE %s.%I OWNER TO %I'
+            , '@extschema@'
+            , v_template_tablename
+            , v_parent_owner);
+ELSIF lower(p_template_table) IN ('false', 'f') THEN
+    v_template_schema := NULL;
+    v_template_tablename := NULL;
+    RAISE DEBUG 'create_parent(): parent_table: %, skipped template table creation', p_parent_table;
 ELSE
-    v_inherit_privileges  = true;
+    SELECT n.nspname, c.relname INTO v_template_schema, v_template_tablename
+    FROM pg_catalog.pg_class c
+    JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
+    WHERE n.nspname = split_part(p_template_table, '.', 1)::name
+    AND c.relname = split_part(p_template_table, '.', 2)::name;
+        IF v_template_tablename IS NULL THEN
+            RAISE EXCEPTION 'Unable to find given template table in system catalogs (%). Please create template table first or leave parameter NULL to have a default one created for you.', p_parent_table;
+        END IF;
 END IF;
 
 SELECT current_setting('search_path') INTO v_old_search_path;
@@ -266,7 +220,7 @@ END IF;
 
 -- If this parent table has siblings that are also partitioned (subpartitions), ensure this parent gets added to part_config_sub table so future maintenance will subpartition it
 -- Just doing in a loop to avoid having to assign a bunch of variables (should only run once, if at all; constraint should enforce only one value.)
-FOR v_row IN 
+FOR v_row IN
     WITH parent_table AS (
         SELECT h.inhparent AS parent_oid
         FROM pg_catalog.pg_inherits h
@@ -275,37 +229,33 @@ FOR v_row IN
         WHERE c.relname = v_parent_tablename::name
         AND n.nspname = v_parent_schema::name
     ), sibling_children AS (
-        SELECT i.inhrelid::regclass::text AS tablename 
+        SELECT i.inhrelid::regclass::text AS tablename
         FROM pg_inherits i
         JOIN parent_table p ON i.inhparent = p.parent_oid
     )
-    -- This column list must be kept consistent between: 
+    -- This column list must be kept consistent between:
     --   create_parent, check_subpart_sameconfig, create_partition_id, create_partition_time, dump_partitioned_table_definition and table definition
-    SELECT DISTINCT sub_partition_type
-        , sub_control
-        , sub_partition_interval
-        , sub_constraint_cols
-        , sub_premake
-        , sub_inherit_fk
-        , sub_retention
-        , sub_retention_schema
-        , sub_retention_keep_table
-        , sub_retention_keep_index
-        , sub_automatic_maintenance
-        , sub_epoch
-        , sub_optimize_trigger
-        , sub_optimize_constraint
-        , sub_infinite_time_partitions
-        , sub_jobmon
-        , sub_trigger_exception_handling
-        , sub_upsert
-        , sub_trigger_return_null
-        , sub_template_table
-        , sub_inherit_privileges
-        , sub_constraint_valid
-        , sub_subscription_refresh
-        , sub_date_trunc_interval
-        , sub_ignore_default_data
+    SELECT DISTINCT
+        a.sub_control
+        , a.sub_partition_interval
+        , a.sub_partition_type
+        , a.sub_premake
+        , a.sub_automatic_maintenance
+        , a.sub_template_table
+        , a.sub_retention
+        , a.sub_retention_schema
+        , a.sub_retention_keep_index
+        , a.sub_retention_keep_table
+        , a.sub_epoch
+        , a.sub_constraint_cols
+        , a.sub_optimize_constraint
+        , a.sub_infinite_time_partitions
+        , a.sub_jobmon
+        , a.sub_inherit_privileges
+        , a.sub_constraint_valid
+        , a.sub_date_trunc_interval
+        , a.sub_ignore_default_data
+        , a.sub_default_table
     FROM @extschema@.part_config_sub a
     JOIN sibling_children b on a.sub_parent = b.tablename LIMIT 1
 LOOP
@@ -316,24 +266,18 @@ LOOP
         , sub_partition_interval
         , sub_constraint_cols
         , sub_premake
-        , sub_inherit_fk
         , sub_retention
         , sub_retention_schema
         , sub_retention_keep_table
         , sub_retention_keep_index
         , sub_automatic_maintenance
         , sub_epoch
-        , sub_optimize_trigger
         , sub_optimize_constraint
         , sub_infinite_time_partitions
         , sub_jobmon
-        , sub_trigger_exception_handling
-        , sub_upsert
-        , sub_trigger_return_null
         , sub_template_table
         , sub_inherit_privileges
         , sub_constraint_valid
-        , sub_subscription_refresh
         , sub_date_trunc_interval
         , sub_ignore_default_data)
     VALUES (
@@ -343,122 +287,41 @@ LOOP
         , v_row.sub_partition_interval
         , v_row.sub_constraint_cols
         , v_row.sub_premake
-        , v_row.sub_inherit_fk
         , v_row.sub_retention
         , v_row.sub_retention_schema
-        , v_row.sub_retention_keep_table
         , v_row.sub_retention_keep_index
+        , v_row.sub_retention_keep_table
         , v_row.sub_automatic_maintenance
         , v_row.sub_epoch
-        , v_row.sub_optimize_trigger
         , v_row.sub_optimize_constraint
         , v_row.sub_infinite_time_partitions
         , v_row.sub_jobmon
-        , v_row.sub_trigger_exception_handling
-        , v_row.sub_upsert
-        , v_row.sub_trigger_return_null
         , v_row.sub_template_table
         , v_row.sub_inherit_privileges
         , v_row.sub_constraint_valid
-        , v_row.sub_subscription_refresh
         , v_row.sub_date_trunc_interval
         , v_row.sub_ignore_default_data);
 
-    -- Set this equal to sibling configs so that newly created child table 
+    -- Set this equal to sibling configs so that newly created child table
     -- privileges are set properly below during initial setup.
-    -- This setting is special because it applies immediately to the new child 
+    -- This setting is special because it applies immediately to the new child
     -- tables of a given parent, not just during maintenance like most other settings.
     v_inherit_privileges = v_row.sub_inherit_privileges;
 END LOOP;
 
 IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
 
-    CASE
-        WHEN p_interval = 'yearly' THEN
-            v_time_interval := '1 year';
-        WHEN p_interval = 'quarterly' THEN
-            v_time_interval := '3 months';
-        WHEN p_interval = 'monthly' THEN
-            v_time_interval := '1 month';
-        WHEN p_interval  = 'weekly' THEN
-            v_time_interval := '1 week';
-        WHEN p_interval = 'daily' THEN
-            v_time_interval := '1 day';
-        WHEN p_interval = 'hourly' THEN
-            v_time_interval := '1 hour';
-        WHEN p_interval = 'half-hour' THEN
-            v_time_interval := '30 mins';
-        WHEN p_interval = 'quarter-hour' THEN
-            v_time_interval := '15 mins';
-        ELSE
-            IF p_type <> 'native' THEN
-                -- Reset for use as part_config type value below
-                p_type = 'time-custom';
-            END IF;
-            v_time_interval := p_interval::interval;
-            IF v_time_interval < '1 second'::interval THEN
-                RAISE EXCEPTION 'Partitioning interval must be 1 second or greater';
-            END IF;
-    END CASE;
+    v_time_interval := p_interval::interval;
+    IF v_time_interval < '1 second'::interval THEN
+        RAISE EXCEPTION 'Partitioning interval must be 1 second or greater';
+    END IF;
 
    -- First partition is either the min premake or p_start_partition
     v_start_time := COALESCE(p_start_partition::timestamptz, CURRENT_TIMESTAMP - (v_time_interval * p_premake));
 
-
-    v_datetime_string := 'YYYY';
-    IF p_date_trunc_interval IS NOT NULL THEN
-
-        v_base_timestamp := date_trunc(p_date_trunc_interval, v_start_time);
-
-        IF v_time_interval >= '1 day' THEN
-            v_datetime_string := v_datetime_string || '_MM_DD';
-        ELSE
-            v_datetime_string := v_datetime_string || '_MM_DD_HH24MISS';
-        END IF;
-
-    ELSE
-
-        IF v_time_interval >= '1 year' THEN
-            v_base_timestamp := date_trunc('year', v_start_time);
-            IF v_time_interval >= '10 years' THEN
-                v_base_timestamp := date_trunc('decade', v_start_time);
-                IF v_time_interval >= '100 years' THEN
-                    v_base_timestamp := date_trunc('century', v_start_time);
-                    IF v_time_interval >= '1000 years' THEN
-                        v_base_timestamp := date_trunc('millennium', v_start_time);
-                    END IF; -- 1000
-                END IF; -- 100
-            END IF; -- 10
-        END IF; -- 1
-
-        IF v_time_interval < '1 year' THEN
-            IF p_interval = 'quarterly' THEN
-                v_base_timestamp := date_trunc('quarter', v_start_time);
-                v_datetime_string = 'YYYY"q"Q';
-            ELSE
-                v_base_timestamp := date_trunc('month', v_start_time); 
-                v_datetime_string := v_datetime_string || '_MM';
-            END IF;
-            IF v_time_interval < '1 month' THEN
-                IF p_interval = 'weekly' THEN
-                    v_base_timestamp := date_trunc('week', v_start_time);
-                    v_datetime_string := 'IYYY"w"IW';
-                ELSE 
-                    v_base_timestamp := date_trunc('day', v_start_time);
-                    v_datetime_string := v_datetime_string || '_DD';
-                END IF;
-                IF v_time_interval < '1 day' THEN
-                    v_base_timestamp := date_trunc('hour', v_start_time);
-                    v_datetime_string := v_datetime_string || '_HH24MI';
-                    IF v_time_interval < '1 minute' THEN
-                        v_base_timestamp := date_trunc('minute', v_start_time);
-                        v_datetime_string := v_datetime_string || 'SS';
-                    END IF; -- minute
-                END IF; -- day
-            END IF; -- month
-        END IF; -- year
-
-    END IF; -- end p_date_trunc_interval IF
+    SELECT base_timestamp, datetime_string
+    INTO v_base_timestamp, v_datetime_string
+    FROM @extschema@.calculate_time_partition_info(v_time_interval, v_start_time, p_date_trunc_interval);
 
     RAISE DEBUG 'create_parent(): parent_table: %, v_base_timestamp: %', p_parent_table, v_base_timestamp;
 
@@ -470,7 +333,7 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
                 v_partition_time := (v_base_timestamp + (v_time_interval * v_count))::timestamptz;
                 v_partition_time_array := array_append(v_partition_time_array, v_partition_time);
             EXCEPTION WHEN datetime_field_overflow THEN
-                RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range. 
+                RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range.
                     Child partition creation after time % skipped', v_partition_time;
                 v_step_overflow_id := add_step(v_job_id, 'Attempted partition time interval is outside PostgreSQL''s supported time range.');
                 PERFORM update_step(v_step_overflow_id, 'CRITICAL', 'Child partition creation after time '||v_partition_time||' skipped');
@@ -492,13 +355,11 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
         , constraint_cols
         , datetime_string
         , automatic_maintenance
-        , inherit_fk
-        , jobmon 
-        , upsert
-        , trigger_return_null
+        , jobmon
         , template_table
-        , publications
-        , inherit_privileges)
+        , inherit_privileges
+        , default_table
+        , date_trunc_interval)
     VALUES (
         p_parent_table
         , p_type
@@ -509,19 +370,17 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
         , p_constraint_cols
         , v_datetime_string
         , p_automatic_maintenance
-        , p_inherit_fk
         , p_jobmon
-        , p_upsert
-        , p_trigger_return_null
         , v_template_schema||'.'||v_template_tablename
-        , p_publications
-        , v_inherit_privileges);
+        , v_inherit_privileges
+        , p_default_table
+        , p_date_trunc_interval);
 
     RAISE DEBUG 'create_parent: v_partition_time_array: %', v_partition_time_array;
 
-    v_last_partition_created := @extschema@.create_partition_time(p_parent_table, v_partition_time_array, false);
+    v_last_partition_created := @extschema@.create_partition_time(p_parent_table, v_partition_time_array);
 
-    IF v_last_partition_created = false THEN 
+    IF v_last_partition_created = false THEN
         -- This can happen with subpartitioning when future or past partitions prevent child creation because they're out of range of the parent
         -- First see if this parent is a subpartition managed by pg_partman
         WITH top_oid AS (
@@ -531,8 +390,8 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
             JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
             WHERE c.relname = v_parent_tablename::name
             AND n.nspname = v_parent_schema::name
-        ) SELECT n.nspname, c.relname 
-        INTO v_top_parent_schema, v_top_parent_table 
+        ) SELECT n.nspname, c.relname
+        INTO v_top_parent_schema, v_top_parent_table
         FROM pg_catalog.pg_class c
         JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
         JOIN top_oid t ON c.oid = t.top_parent_oid
@@ -554,7 +413,7 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
             END IF;
             v_partition_time_array := NULL;
             v_partition_time_array := array_append(v_partition_time_array, v_base_timestamp);
-            v_last_partition_created := @extschema@.create_partition_time(p_parent_table, v_partition_time_array, false);
+            v_last_partition_created := @extschema@.create_partition_time(p_parent_table, v_partition_time_array);
         ELSE
             RAISE WARNING 'No child tables created. Check that all child tables did not already exist and may not have been part of partition set. Given parent has still been configured with pg_partman, but may not have expected children. Please review schema and config to confirm things are ok.';
 
@@ -570,7 +429,7 @@ IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
             EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
 
             RETURN v_success;
-        END IF; 
+        END IF;
     END IF; -- End v_last_partition IF
 
     IF v_jobmon_schema IS NOT NULL THEN
@@ -581,11 +440,12 @@ END IF;
 
 IF v_control_type = 'id' AND p_epoch = 'none' THEN
     v_id_interval := p_interval::bigint;
-    IF p_type <> 'native' AND v_id_interval < 10 THEN
-        RAISE EXCEPTION 'Interval for serial, non-native partitioning must be greater than or equal to 10';
-    END IF;
+    -- TODO: When list partitioning is supported, do not support interval less than 2 for ranged
+    -- IF v_id_interval < 2 THEN
+    --    RAISE EXCEPTION 'Interval for range partitioning must be greater than or equal to 2. Use LIST partitioning for single value partitions.';
+    -- END IF;
 
-    -- Check if parent table is a subpartition of an already existing id partition set managed by pg_partman. 
+    -- Check if parent table is a subpartition of an already existing id partition set managed by pg_partman.
     WHILE v_higher_parent_table IS NOT NULL LOOP -- initially set in DECLARE
         WITH top_oid AS (
             SELECT i.inhparent AS top_parent_oid
@@ -631,12 +491,12 @@ IF v_control_type = 'id' AND p_epoch = 'none' THEN
     v_starting_partition_id := v_max - (v_max % v_id_interval);
     FOR i IN 0..p_premake LOOP
         -- Only make previous partitions if ID value is less than the starting value and positive (and custom start partition wasn't set)
-        IF p_start_partition IS NULL AND 
-            (v_starting_partition_id - (v_id_interval*i)) > 0 AND 
-            (v_starting_partition_id - (v_id_interval*i)) < v_starting_partition_id 
+        IF p_start_partition IS NULL AND
+            (v_starting_partition_id - (v_id_interval*i)) > 0 AND
+            (v_starting_partition_id - (v_id_interval*i)) < v_starting_partition_id
         THEN
             v_partition_id_array = array_append(v_partition_id_array, (v_starting_partition_id - v_id_interval*i));
-        END IF; 
+        END IF;
         v_partition_id_array = array_append(v_partition_id_array, (v_id_interval*i) + v_starting_partition_id);
     END LOOP;
 
@@ -648,13 +508,11 @@ IF v_control_type = 'id' AND p_epoch = 'none' THEN
         , premake
         , constraint_cols
         , automatic_maintenance
-        , inherit_fk
         , jobmon
-        , upsert
-        , trigger_return_null
         , template_table
-        , publications
-        , inherit_privileges)
+        , inherit_privileges
+        , default_table
+        , date_trunc_interval)
     VALUES (
         p_parent_table
         , p_type
@@ -662,16 +520,14 @@ IF v_control_type = 'id' AND p_epoch = 'none' THEN
         , p_control
         , p_premake
         , p_constraint_cols
-        , p_automatic_maintenance 
-        , p_inherit_fk
+        , p_automatic_maintenance
         , p_jobmon
-        , p_upsert
-        , p_trigger_return_null
         , v_template_schema||'.'||v_template_tablename
-        , p_publications
-        , v_inherit_privileges); 
+        , v_inherit_privileges
+        , p_default_table
+        , p_date_trunc_interval);
 
-    v_last_partition_created := @extschema@.create_partition_id(p_parent_table, v_partition_id_array, false);
+    v_last_partition_created := @extschema@.create_partition_id(p_parent_table, v_partition_id_array);
 
     IF v_last_partition_created = false THEN
         -- This can happen with subpartitioning when future or past partitions prevent child creation because they're out of range of the parent
@@ -706,7 +562,7 @@ IF v_control_type = 'id' AND p_epoch = 'none' THEN
             END IF;
             v_partition_id_array = NULL;
             v_partition_id_array = array_append(v_partition_id_array, v_starting_partition_id);
-            v_last_partition_created := @extschema@.create_partition_id(p_parent_table, v_partition_id_array, false);
+            v_last_partition_created := @extschema@.create_partition_id(p_parent_table, v_partition_id_array);
         ELSE
             -- Currently unknown edge case if code gets here
             RAISE WARNING 'No child tables created. Check that all child tables did not already exist and may not have been part of partition set. Given parent has still been configured with pg_partman, but may not have expected children. Please review schema and config to confirm things are ok.';
@@ -727,70 +583,34 @@ IF v_control_type = 'id' AND p_epoch = 'none' THEN
 
 END IF; -- End IF id
 
-IF p_type = 'native' AND current_setting('server_version_num')::int >= 110000 THEN
-    -- Add default partition to native sets in PG11+
+IF p_default_table THEN
+    -- Add default partition
 
     v_default_partition := @extschema@.check_name_length(v_parent_tablename, '_default', FALSE);
-    v_sql := 'CREATE'; 
+    v_sql := 'CREATE';
 
-    -- Left this here as reminder to revisit once native figures out how it is handling changing unlogged stats
+    -- Left this here as reminder to revisit once core PG figures out how it is handling changing unlogged stats
     -- Currently handed via template table below
-    /* 
+    /*
     IF v_unlogged = 'u' THEN
          v_sql := v_sql ||' UNLOGGED';
     END IF;
     */
 
     -- Same INCLUDING list is used in create_partition_*(). INDEXES is handled when partition is attached if it's supported.
-    v_sql := v_sql || format(' TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS '
+    v_sql := v_sql || format(' TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS INCLUDING GENERATED)'
         , v_parent_schema, v_default_partition, v_parent_schema, v_parent_tablename);
-    IF current_setting('server_version_num')::int >= 120000 THEN
-        v_sql := v_sql || ' INCLUDING GENERATED ';
-    END IF;
-    v_sql := v_sql || ')';
     EXECUTE v_sql;
+
     v_sql := format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I DEFAULT'
         , v_parent_schema, v_parent_tablename, v_parent_schema, v_default_partition);
     EXECUTE v_sql;
 
-    IF p_publications IS NOT NULL THEN
-        -- NOTE: Native publication inheritance is only supported on PG14+
-        PERFORM @extschema@.apply_publications(p_parent_table, v_parent_schema, v_default_partition);
-    END IF;
-
-
-    IF current_setting('server_version_num')::int >= 120000 AND v_parent_tablespace IS NOT NULL THEN
-        -- Tablespace managed via inherit_template_properties() call below if PG11 or earliser
-        EXECUTE format('ALTER TABLE %I.%I SET TABLESPACE %I', v_parent_schema, v_default_partition, v_parent_tablespace);
-    END IF;
-
-    -- Manage template inherited properies
+    -- Manage template inherited properties
     PERFORM @extschema@.inherit_template_properties(p_parent_table, v_parent_schema, v_default_partition);
 
 END IF;
 
-IF p_type <> 'native' THEN
-    IF v_jobmon_schema IS NOT NULL  THEN
-        v_step_id := add_step(v_job_id, 'Creating partition function');
-    END IF;
-    IF v_control_type = 'time' OR (v_control_type = 'id' AND p_epoch <> 'none') THEN
-        PERFORM @extschema@.create_function_time(p_parent_table, v_job_id);
-        IF v_jobmon_schema IS NOT NULL THEN
-            PERFORM update_step(v_step_id, 'OK', 'Time function created');
-        END IF;
-    ELSIF v_control_type = 'id' THEN
-        PERFORM @extschema@.create_function_id(p_parent_table, v_job_id);  
-        IF v_jobmon_schema IS NOT NULL THEN
-            PERFORM update_step(v_step_id, 'OK', 'ID function created');
-        END IF;
-    END IF;
-
-    IF v_jobmon_schema IS NOT NULL THEN
-        v_step_id := add_step(v_job_id, 'Creating partition trigger');
-    END IF;
-    PERFORM @extschema@.create_trigger(p_parent_table);
-END IF; -- end native check
-
 
 IF v_jobmon_schema IS NOT NULL THEN
     PERFORM update_step(v_step_id, 'OK', 'Done');
@@ -829,4 +649,3 @@ DETAIL: %
 HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
-
