@@ -1,5 +1,10 @@
-CREATE FUNCTION @extschema@.create_partition_id(p_parent_table text, p_partition_ids bigint[], p_analyze boolean DEFAULT true, p_start_partition text DEFAULT NULL) RETURNS boolean
-    LANGUAGE plpgsql 
+CREATE FUNCTION @extschema@.create_partition_id(
+    p_parent_table text
+    , p_partition_ids bigint[]
+    , p_start_partition text DEFAULT NULL
+)
+    RETURNS boolean
+    LANGUAGE plpgsql
     AS $$
 DECLARE
 
@@ -7,40 +12,29 @@ ex_context              text;
 ex_detail               text;
 ex_hint                 text;
 ex_message              text;
-v_all                   text[] := ARRAY['SELECT', 'INSERT', 'UPDATE', 'DELETE', 'TRUNCATE', 'REFERENCES', 'TRIGGER'];
-v_analyze               boolean := FALSE;
 v_control               text;
 v_control_type          text;
 v_exists                text;
-v_grantees              text[];
-v_hasoids               boolean;
 v_id                    bigint;
-v_inherit_fk            boolean;
 v_inherit_privileges    boolean;
 v_job_id                bigint;
 v_jobmon                boolean;
 v_jobmon_schema         text;
 v_new_search_path       text;
 v_old_search_path       text;
-v_parent_grant          record;
 v_parent_schema         text;
 v_parent_tablename      text;
-v_parent_tablespace     text;
 v_partition_interval    bigint;
 v_partition_created     boolean := false;
 v_partition_name        text;
-v_partition_type        text;
-v_publications          text[];
-v_revoke                text;
 v_row                   record;
 v_sql                   text;
 v_step_id               bigint;
 v_sub_control           text;
-v_sub_partition_type    text; 
+v_sub_partition_type    text;
 v_sub_id_max            bigint;
 v_sub_id_min            bigint;
 v_template_table        text;
-v_unlogged              char;
 
 BEGIN
 /*
@@ -48,20 +42,14 @@ BEGIN
  */
 
 SELECT control
-    , partition_type
-    , partition_interval
-    , inherit_fk
+    , partition_interval::bigint -- this shared field also used in partition_time as interval
     , jobmon
     , template_table
-    , publications
     , inherit_privileges
 INTO v_control
-    , v_partition_type
     , v_partition_interval
-    , v_inherit_fk
     , v_jobmon
     , v_template_table
-    , v_publications
     , v_inherit_privileges
 FROM @extschema@.part_config
 WHERE parent_table = p_parent_table;
@@ -70,11 +58,10 @@ IF NOT FOUND THEN
     RAISE EXCEPTION 'ERROR: no config found for %', p_parent_table;
 END IF;
 
-SELECT n.nspname, c.relname, t.spcname 
-INTO v_parent_schema, v_parent_tablename, v_parent_tablespace 
+SELECT n.nspname, c.relname
+INTO v_parent_schema, v_parent_tablename
 FROM pg_catalog.pg_class c
 JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-LEFT OUTER JOIN pg_catalog.pg_tablespace t ON c.reltablespace = t.oid
 WHERE n.nspname = split_part(p_parent_table, '.', 1)::name
 
 AND c.relname = split_part(p_parent_table, '.', 2)::name;
@@ -106,7 +93,7 @@ IF v_jobmon_schema IS NOT NULL THEN
 END IF;
 
 FOREACH v_id IN ARRAY p_partition_ids LOOP
--- Do not create the child table if it's outside the bounds of the top parent. 
+-- Do not create the child table if it's outside the bounds of the top parent.
     IF v_sub_id_min IS NOT NULL THEN
         IF v_id < v_sub_id_min OR v_id >= v_sub_id_max THEN
             CONTINUE;
@@ -116,7 +103,7 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
     v_partition_name := @extschema@.check_name_length(v_parent_tablename, v_id::text, TRUE);
     -- If child table already exists, skip creation
     -- Have to check pg_class because if subpartitioned, table will not be in pg_tables
-    SELECT c.relname INTO v_exists 
+    SELECT c.relname INTO v_exists
     FROM pg_catalog.pg_class c
     JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
     WHERE n.nspname = v_parent_schema::name AND c.relname = v_partition_name::name;
@@ -124,116 +111,45 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
         CONTINUE;
     END IF;
 
-    -- Ensure analyze is run if a new partition is created. Otherwise if one isn't, will be false and analyze will be skipped
-    v_analyze := TRUE;
-
     IF v_jobmon_schema IS NOT NULL THEN
         v_step_id := add_step(v_job_id, 'Creating new partition '||v_partition_name||' with interval from '||v_id||' to '||(v_id + v_partition_interval)-1);
     END IF;
 
-    v_sql := 'CREATE';
-
-    -- As of PG12, the unlogged/logged status of a native parent table cannot be changed via an ALTER TABLE in order to affect its children.
-    -- As of v4.2x, the unlogged state will be managed via the template table    
-    SELECT relpersistence INTO v_unlogged 
-    FROM pg_catalog.pg_class c
-    JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-    WHERE c.relname = v_parent_tablename::name
-    AND n.nspname = v_parent_schema::name;
-    IF v_unlogged = 'u' and v_partition_type != 'native'  THEN
-        v_sql := v_sql || ' UNLOGGED';
-    END IF;
-
-    -- Close parentheses on LIKE are below due to differing requirements of native subpartitioning
+    -- Close parentheses on LIKE are below due to differing requirements of subpartitioning
     -- Same INCLUDING list is used in create_parent()
-    v_sql := v_sql || format(' TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS '
+    v_sql := format('CREATE TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS INCLUDING GENERATED '
             , v_parent_schema
             , v_partition_name
             , v_parent_schema
             , v_parent_tablename);
 
-    IF current_setting('server_version_num')::int >= 120000 THEN
-        v_sql := v_sql || ' INCLUDING GENERATED ';
-    END IF;
-
-    SELECT sub_partition_type, sub_control INTO v_sub_partition_type, v_sub_control 
-    FROM @extschema@.part_config_sub 
+    SELECT sub_partition_type, sub_control INTO v_sub_partition_type, v_sub_control
+    FROM @extschema@.part_config_sub
     WHERE sub_parent = p_parent_table;
-    IF v_sub_partition_type = 'native' THEN
-        -- INCLUDING INDEXES isn't necessary for native partitioning. It isn't supported in v10 and 
-	    --   for v11+ index inheritance is automatically handled when the partition is attached
+    IF v_sub_partition_type = 'range' THEN
         v_sql := v_sql || format(') PARTITION BY RANGE (%I) ', v_sub_control);
     ELSE
-        v_sql := v_sql || format(' INCLUDING INDEXES) ', v_sub_control);
-    END IF;
-
-
-    IF current_setting('server_version_num')::int < 120000 THEN
-        -- column removed from pgclass in pg12
-        SELECT relhasoids INTO v_hasoids 
-        FROM pg_catalog.pg_class c
-        JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE c.relname = v_parent_tablename::name
-        AND n.nspname = v_parent_schema::name;
-        IF v_hasoids IS TRUE THEN
-            v_sql := v_sql || ' WITH (OIDS)';
-        END IF;
+        v_sql := v_sql || format(')');
     END IF;
 
     RAISE DEBUG 'create_partition_id v_sql: %', v_sql;
     EXECUTE v_sql;
 
-    IF v_partition_type = 'native' THEN
-
-        IF current_setting('server_version_num')::int >= 120000 THEN
-            -- PG12 fixed tablespace marking on the parent of a native partition set
-            -- Versions older than 12 handle tablespace setting via inherit_template_properties() call below
-            IF v_parent_tablespace IS NOT NULL THEN
-                EXECUTE format('ALTER TABLE %I.%I SET TABLESPACE %I', v_parent_schema, v_partition_name, v_parent_tablespace);
-            END IF;
-        END IF;
-
-        IF v_template_table IS NOT NULL THEN
-            PERFORM @extschema@.inherit_template_properties(p_parent_table, v_parent_schema, v_partition_name);
-        END IF;
-
-        EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
-            , v_parent_schema
-            , v_parent_tablename
-            , v_parent_schema
-            , v_partition_name
-            , v_id
-            , v_id + v_partition_interval);
-
-    ELSE -- non-native
-
-        IF v_parent_tablespace IS NOT NULL THEN
-            EXECUTE format('ALTER TABLE %I.%I SET TABLESPACE %I', v_parent_schema, v_partition_name, v_parent_tablespace);
-        END IF;
-
-        EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%I >= %s AND %I < %s )'
-            , v_parent_schema
-            , v_partition_name
-            , v_partition_name||'_partition_check'
-            , v_control
-            , v_id
-            , v_control
-            , v_id + v_partition_interval);
-
-        EXECUTE format('ALTER TABLE %I.%I INHERIT %I.%I', v_parent_schema, v_partition_name, v_parent_schema, v_parent_tablename);
-
-        -- Indexes cannot be created on the parent, so clustering cannot be used for native yet.
-        PERFORM @extschema@.apply_cluster(v_parent_schema, v_parent_tablename, v_parent_schema, v_partition_name);
-
-        -- Foreign keys to other tables not supported on native parent tables
-        IF v_inherit_fk THEN
-            PERFORM @extschema@.apply_foreign_keys(p_parent_table, v_parent_schema||'.'||v_partition_name, v_job_id);
-        END IF;
-
+    IF v_template_table IS NOT NULL THEN
+        PERFORM @extschema@.inherit_template_properties(p_parent_table, v_parent_schema, v_partition_name);
     END IF;
-    
-    -- NOTE: Privileges not automatically inherited for native. Only do so if config flag is set
-    IF v_partition_type != 'native' OR (v_partition_type = 'native' AND v_inherit_privileges = TRUE) THEN
+
+    EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
+        , v_parent_schema
+        , v_parent_tablename
+        , v_parent_schema
+        , v_partition_name
+        , v_id
+        , v_id + v_partition_interval);
+
+
+    -- NOTE: Privileges not automatically inherited. Only do so if config flag is set
+    IF v_inherit_privileges = TRUE THEN
         PERFORM @extschema@.apply_privileges(v_parent_schema, v_parent_tablename, v_parent_schema, v_partition_name, v_job_id);
     END IF;
 
@@ -243,35 +159,31 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
 
     -- Will only loop once and only if sub_partitioning is actually configured
     -- This seemed easier than assigning a bunch of variables then doing an IF condition
-    -- This column list must be kept consistent between: 
+    -- This column list must be kept consistent between:
     --   create_parent, check_subpart_sameconfig, create_partition_id, create_partition_time, dump_partitioned_table_definition, and table definition
-    FOR v_row IN 
-        SELECT sub_parent
-            , sub_partition_type
+    FOR v_row IN
+        SELECT
+            sub_parent
             , sub_control
             , sub_partition_interval
-            , sub_constraint_cols
+            , sub_partition_type
             , sub_premake
-            , sub_optimize_trigger
-            , sub_optimize_constraint
-            , sub_epoch
-            , sub_inherit_fk
+            , sub_automatic_maintenance
+            , sub_template_table
             , sub_retention
             , sub_retention_schema
-            , sub_retention_keep_table
             , sub_retention_keep_index
+            , sub_retention_keep_table
+            , sub_epoch
+            , sub_constraint_cols
+            , sub_optimize_constraint
             , sub_infinite_time_partitions
-            , sub_automatic_maintenance
             , sub_jobmon
-            , sub_trigger_exception_handling
-            , sub_upsert
-            , sub_trigger_return_null
-            , sub_template_table
             , sub_inherit_privileges
             , sub_constraint_valid
-            , sub_subscription_refresh
             , sub_date_trunc_interval
             , sub_ignore_default_data
+            , sub_default_table
         FROM @extschema@.part_config_sub
         WHERE sub_parent = p_parent_table
     LOOP
@@ -283,23 +195,23 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
                 , p_control := %L
                 , p_type := %L
                 , p_interval := %L
+                , p_default_table := %L
                 , p_constraint_cols := %L
                 , p_premake := %L
                 , p_automatic_maintenance := %L
-                , p_inherit_fk := %L
                 , p_epoch := %L
                 , p_template_table := %L
                 , p_jobmon := %L
-                , p_start_partition := %L 
+                , p_start_partition := %L
                 , p_date_trunc_interval := %L )'
             , v_parent_schema||'.'||v_partition_name
             , v_row.sub_control
             , v_row.sub_partition_type
             , v_row.sub_partition_interval
+            , v_row.sub_default_table
             , v_row.sub_constraint_cols
             , v_row.sub_premake
             , v_row.sub_automatic_maintenance
-            , v_row.sub_inherit_fk
             , v_row.sub_epoch
             , v_row.sub_template_table
             , v_row.sub_jobmon
@@ -308,19 +220,13 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
         RAISE DEBUG 'create_partition_id (create_parent loop): %', v_sql;
         EXECUTE v_sql;
 
-        UPDATE @extschema@.part_config SET 
+        UPDATE @extschema@.part_config SET
             retention_schema = v_row.sub_retention_schema
             , retention_keep_table = v_row.sub_retention_keep_table
-            , retention_keep_index = v_row.sub_retention_keep_index
-            , optimize_trigger = v_row.sub_optimize_trigger
             , optimize_constraint = v_row.sub_optimize_constraint
             , infinite_time_partitions = v_row.sub_infinite_time_partitions
-            , trigger_exception_handling = v_row.sub_trigger_exception_handling
-            , upsert = v_row.sub_upsert
             , inherit_privileges = v_row.sub_inherit_privileges
-            , trigger_return_null = v_row.sub_trigger_return_null
             , constraint_valid = v_row.sub_constraint_valid
-            , subscription_refresh = v_row.sub_subscription_refresh
             , ignore_default_data = v_row.sub_ignore_default_data
         WHERE parent_table = v_parent_schema||'.'||v_partition_name;
 
@@ -329,14 +235,9 @@ FOREACH v_id IN ARRAY p_partition_ids LOOP
         END IF;
 
     END LOOP; -- end sub partitioning LOOP
-    
-    -- Manage additonal constraints if set
-    PERFORM @extschema@.apply_constraints(p_parent_table, p_job_id := v_job_id);
 
-    IF v_publications IS NOT NULL THEN
-        -- NOTE: Native publication inheritance is only supported on PG14+
-        PERFORM @extschema@.apply_publications(p_parent_table, v_parent_schema, v_partition_name);
-    END IF;
+    -- Manage additional constraints if set
+    PERFORM @extschema@.apply_constraints(p_parent_table, p_job_id := v_job_id);
 
     v_partition_created := true;
 
@@ -344,6 +245,7 @@ END LOOP;
 
 -- v_analyze is a local check if a new table is made.
 -- p_analyze is a parameter to say whether to run the analyze at all. Used by create_parent() to avoid long exclusive lock or run_maintenence() to avoid long creation runs.
+/** REMOVE
 IF v_analyze AND p_analyze THEN
     IF v_jobmon_schema IS NOT NULL THEN
         v_step_id := add_step(v_job_id, format('Analyzing partition set: %s', p_parent_table));
@@ -355,6 +257,7 @@ IF v_analyze AND p_analyze THEN
         PERFORM update_step(v_step_id, 'OK', 'Done');
     END IF;
 END IF;
+REMOVE **/
 
 IF v_jobmon_schema IS NOT NULL THEN
     IF v_partition_created = false THEN
@@ -364,7 +267,7 @@ IF v_jobmon_schema IS NOT NULL THEN
 
     PERFORM close_job(v_job_id);
 END IF;
- 
+
 EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
 
 RETURN v_partition_created;
@@ -388,7 +291,6 @@ EXCEPTION
         RAISE EXCEPTION '%
 CONTEXT: %
 DETAIL: %
-HINT: %', ex_message, ex_context, ex_detail, ex_hint; 
+HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
-
