@@ -1,5 +1,9 @@
-CREATE FUNCTION @extschema@.create_partition_time(p_parent_table text, p_partition_times timestamptz[], p_analyze boolean DEFAULT true, p_start_partition text DEFAULT NULL) 
-RETURNS boolean
+CREATE FUNCTION @extschema@.create_partition_time(
+    p_parent_table text
+    , p_partition_times timestamptz[]
+    , p_start_partition text DEFAULT NULL
+)
+    RETURNS boolean
     LANGUAGE plpgsql
     AS $$
 DECLARE
@@ -8,77 +12,55 @@ ex_context                      text;
 ex_detail                       text;
 ex_hint                         text;
 ex_message                      text;
-v_all                           text[] := ARRAY['SELECT', 'INSERT', 'UPDATE', 'DELETE', 'TRUNCATE', 'REFERENCES', 'TRIGGER'];
-v_analyze                       boolean := FALSE;
 v_control                       text;
 v_control_type                  text;
 v_datetime_string               text;
 v_epoch                         text;
 v_exists                        smallint;
-v_grantees                      text[];
-v_hasoids                       boolean;
 v_inherit_privileges            boolean;
-v_inherit_fk                    boolean;
 v_job_id                        bigint;
 v_jobmon                        boolean;
 v_jobmon_schema                 text;
 v_new_search_path               text;
 v_old_search_path               text;
-v_parent_grant                  record;
 v_parent_schema                 text;
 v_parent_tablename              text;
-v_part_col                      text;
 v_partition_created             boolean := false;
 v_partition_name                text;
 v_partition_suffix              text;
-v_parent_tablespace             text;
 v_partition_expression          text;
 v_partition_interval            interval;
 v_partition_timestamp_end       timestamptz;
 v_partition_timestamp_start     timestamptz;
-v_publications                  text[];
-v_quarter                       text;
-v_revoke                        text;
 v_row                           record;
 v_sql                           text;
 v_step_id                       bigint;
 v_step_overflow_id              bigint;
 v_sub_control                   text;
-v_sub_parent                    text;
 v_sub_partition_type            text;
 v_sub_timestamp_max             timestamptz;
 v_sub_timestamp_min             timestamptz;
 v_template_table                text;
-v_trunc_value                   text;
 v_time                          timestamptz;
-v_partition_type                          text;
-v_unlogged                      char;
-v_year                          text;
 
 BEGIN
 /*
  * Function to create a child table in a time-based partition set
  */
 
-SELECT partition_type
-    , control
-    , partition_interval
+SELECT control
+    , partition_interval::interval -- this shared field also used in partition_id as bigint
     , epoch
-    , inherit_fk
     , jobmon
     , datetime_string
     , template_table
-    , publications
     , inherit_privileges
-INTO v_partition_type
-    , v_control
+INTO v_control
     , v_partition_interval
     , v_epoch
-    , v_inherit_fk
     , v_jobmon
     , v_datetime_string
     , v_template_table
-    , v_publications
     , v_inherit_privileges
 FROM @extschema@.part_config
 WHERE parent_table = p_parent_table;
@@ -87,16 +69,15 @@ IF NOT FOUND THEN
     RAISE EXCEPTION 'ERROR: no config found for %', p_parent_table;
 END IF;
 
-SELECT n.nspname, c.relname, t.spcname 
-INTO v_parent_schema, v_parent_tablename, v_parent_tablespace 
+SELECT n.nspname, c.relname
+INTO v_parent_schema, v_parent_tablename
 FROM pg_catalog.pg_class c
 JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-LEFT OUTER JOIN pg_catalog.pg_tablespace t ON c.reltablespace = t.oid
 WHERE n.nspname = split_part(p_parent_table, '.', 1)::name
 AND c.relname = split_part(p_parent_table, '.', 2)::name;
 
 SELECT general_type INTO v_control_type FROM @extschema@.check_control_type(v_parent_schema, v_parent_tablename, v_control);
-IF v_control_type <> 'time' THEN 
+IF v_control_type <> 'time' THEN
     IF (v_control_type = 'id' AND v_epoch = 'none') OR v_control_type <> 'id' THEN
         RAISE EXCEPTION 'Cannot run on partition set without time based control column or epoch flag set with an id column. Found control: %, epoch: %', v_control_type, v_epoch;
     END IF;
@@ -131,12 +112,12 @@ v_partition_expression := CASE
 END;
 RAISE DEBUG 'create_partition_time: v_partition_expression: %', v_partition_expression;
 
-FOREACH v_time IN ARRAY p_partition_times LOOP    
+FOREACH v_time IN ARRAY p_partition_times LOOP
     v_partition_timestamp_start := v_time;
     BEGIN
         v_partition_timestamp_end := v_time + v_partition_interval;
     EXCEPTION WHEN datetime_field_overflow THEN
-        RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range. 
+        RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range.
             Child partition creation after time % skipped', v_time;
         v_step_overflow_id := add_step(v_job_id, 'Attempted partition time interval is outside PostgreSQL''s supported time range.');
         PERFORM update_step(v_step_overflow_id, 'CRITICAL', 'Child partition creation after time '||v_time||' skipped');
@@ -144,7 +125,7 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
         CONTINUE;
     END;
 
-    -- Do not create the child table if it's outside the bounds of the top parent. 
+    -- Do not create the child table if it's outside the bounds of the top parent.
     IF v_sub_timestamp_min IS NOT NULL THEN
         IF v_time < v_sub_timestamp_min OR v_time >= v_sub_timestamp_max THEN
 
@@ -158,20 +139,17 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
     -- This suffix generation code is in partition_data_time() as well
     v_partition_suffix := to_char(v_time, v_datetime_string);
     v_partition_name := @extschema@.check_name_length(v_parent_tablename, v_partition_suffix, TRUE);
-    -- Check if child exists. 
+    -- Check if child exists.
     SELECT count(*) INTO v_exists
     FROM pg_catalog.pg_class c
     JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-    WHERE n.nspname = v_parent_schema::name 
+    WHERE n.nspname = v_parent_schema::name
     AND c.relname = v_partition_name::name;
 
     IF v_exists > 0 THEN
         CONTINUE;
     END IF;
 
-    -- Ensure analyze is run if a new partition is created. Otherwise if one isn't, will be false and analyze will be skipped
-    v_analyze := TRUE;
-
     IF v_jobmon_schema IS NOT NULL THEN
         v_step_id := add_step(v_job_id, format('Creating new partition %s.%s with interval from %s to %s'
                                                 , v_parent_schema
@@ -182,123 +160,82 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
 
     v_sql := 'CREATE';
 
-    -- As of PG12, the unlogged/logged status of a native parent table cannot be changed via an ALTER TABLE in order to affect its children.
-    -- As of v4.2x, the unlogged state will be managed via the template table    
-    SELECT relpersistence INTO v_unlogged 
+    /*
+    -- As of PG12, the unlogged/logged status of a parent table cannot be changed via an ALTER TABLE in order to affect its children.
+    -- As of partman v4.2x, the unlogged state will be managed via the template table
+    -- TODO Test UNLOGGED status in PG16 to see if this can be done without template yet. Add to create_partition_id then as well.
+    SELECT relpersistence INTO v_unlogged
     FROM pg_catalog.pg_class c
     JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
     WHERE c.relname = v_parent_tablename::name
     AND n.nspname = v_parent_schema::name;
-    IF v_unlogged = 'u' and v_partition_type != 'native'  THEN
+
+    IF v_unlogged = 'u' THEN
         v_sql := v_sql || ' UNLOGGED';
     END IF;
+    */
 
-    -- Close parentheses on LIKE are below due to differing requirements of native subpartitioning
+    -- Close parentheses on LIKE are below due to differing requirements of subpartitioning
     -- Same INCLUDING list is used in create_parent()
-    v_sql := v_sql || format(' TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS '
+    v_sql := v_sql || format(' TABLE %I.%I (LIKE %I.%I INCLUDING DEFAULTS INCLUDING CONSTRAINTS INCLUDING STORAGE INCLUDING COMMENTS INCLUDING GENERATED '
                                 , v_parent_schema
                                 , v_partition_name
                                 , v_parent_schema
                                 , v_parent_tablename);
 
-    IF current_setting('server_version_num')::int >= 120000 THEN
-        v_sql := v_sql || ' INCLUDING GENERATED ';
-    END IF;
-
-
-    SELECT sub_partition_type, sub_control INTO v_sub_partition_type, v_sub_control 
-    FROM @extschema@.part_config_sub 
+    SELECT sub_partition_type, sub_control INTO v_sub_partition_type, v_sub_control
+    FROM @extschema@.part_config_sub
     WHERE sub_parent = p_parent_table;
-    IF v_sub_partition_type = 'native' THEN
-        -- INCLUDING INDEXES isn't necessary for native partitioning. It isn't supported in v10 and 
-	--   for v11+ index inheritance is automatically handled when the partition is attached
+    IF v_sub_partition_type = 'range' THEN
         v_sql := v_sql || format(') PARTITION BY RANGE (%I) ', v_sub_control);
     ELSE
-        v_sql := v_sql || format(' INCLUDING INDEXES) ');
-    END IF;
-
-    IF current_setting('server_version_num')::int < 120000 THEN
-        -- column removed from pgclass in pg12
-        SELECT relhasoids INTO v_hasoids 
-        FROM pg_catalog.pg_class c
-        JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE c.relname = v_parent_tablename::name
-        AND n.nspname = v_parent_schema::name;
-        IF v_hasoids IS TRUE THEN
-            v_sql := v_sql || ' WITH (OIDS)';
-        END IF;
+        v_sql := v_sql || format(')');
     END IF;
 
     RAISE DEBUG 'create_partition_time v_sql: %', v_sql;
     EXECUTE v_sql;
 
+    IF v_template_table IS NOT NULL THEN
+        PERFORM @extschema@.inherit_template_properties(p_parent_table, v_parent_schema, v_partition_name);
+    END IF;
 
-    IF v_partition_type = 'native' THEN
-
-        IF current_setting('server_version_num')::int >= 120000 THEN
-            -- PG12 fixed tablespace marking on the parent of a native partition set
-            -- Versions older than 12 handle tablespace setting via inherit_template_properties() call below
-            IF v_parent_tablespace IS NOT NULL THEN
-                EXECUTE format('ALTER TABLE %I.%I SET TABLESPACE %I', v_parent_schema, v_partition_name, v_parent_tablespace);
-            END IF;
-        END IF;
-
-        IF v_template_table IS NOT NULL THEN
-            PERFORM @extschema@.inherit_template_properties(p_parent_table, v_parent_schema, v_partition_name);
-        END IF;
-
-        IF v_epoch = 'none' THEN
-            -- Attach with normal, time-based values for native constraint
+    IF v_epoch = 'none' THEN
+        -- Attach with normal, time-based values for built-in constraint
+        EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
+            , v_parent_schema
+            , v_parent_tablename
+            , v_parent_schema
+            , v_partition_name
+            , v_partition_timestamp_start
+            , v_partition_timestamp_end);
+    ELSE
+        -- Must attach with integer based values for built-in constraint and epoch
+        IF v_epoch = 'seconds' THEN
             EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
                 , v_parent_schema
                 , v_parent_tablename
                 , v_parent_schema
                 , v_partition_name
-                , v_partition_timestamp_start
-                , v_partition_timestamp_end);
-        ELSE
-            -- Must attach with integer based values for native constraint and epoch
-            IF v_epoch = 'seconds' THEN
-                EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
-                    , v_parent_schema
-                    , v_parent_tablename
-                    , v_parent_schema
-                    , v_partition_name
-                    , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint
-                    , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint);
-            ELSIF v_epoch = 'milliseconds' THEN
-                EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
-                    , v_parent_schema
-                    , v_parent_tablename
-                    , v_parent_schema
-                    , v_partition_name
-                    , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint * 1000
-                    , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint * 1000);
-            ELSIF v_epoch = 'nanoseconds' THEN
-                EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
-                    , v_parent_schema
-                    , v_parent_tablename
-                    , v_parent_schema
-                    , v_partition_name
-                    , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint * 1000000000
-                    , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint * 1000000000);
-            END IF;
-            -- Create secondary, time-based constraint since native's constraint is already integer based
-            EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%s >= %L AND %4$s < %6$L)'
+                , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint
+                , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint);
+        ELSIF v_epoch = 'milliseconds' THEN
+            EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
+                , v_parent_schema
+                , v_parent_tablename
                 , v_parent_schema
                 , v_partition_name
-                , v_partition_name||'_partition_check'
-                , v_partition_expression
-                , v_partition_timestamp_start
-                , v_partition_timestamp_end);
+                , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint * 1000
+                , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint * 1000);
+        ELSIF v_epoch = 'nanoseconds' THEN
+            EXECUTE format('ALTER TABLE %I.%I ATTACH PARTITION %I.%I FOR VALUES FROM (%L) TO (%L)'
+                , v_parent_schema
+                , v_parent_tablename
+                , v_parent_schema
+                , v_partition_name
+                , EXTRACT('epoch' FROM v_partition_timestamp_start)::bigint * 1000000000
+                , EXTRACT('epoch' FROM v_partition_timestamp_end)::bigint * 1000000000);
         END IF;
-    ELSE -- non-native
-
-        IF v_parent_tablespace IS NOT NULL THEN
-            EXECUTE format('ALTER TABLE %I.%I SET TABLESPACE %I', v_parent_schema, v_partition_name, v_parent_tablespace);
-        END IF;
-
-        -- Non-native always gets time-based constraint
+        -- Create secondary, time-based constraint since built-in's constraint is already integer based
         EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%s >= %L AND %4$s < %6$L)'
             , v_parent_schema
             , v_partition_name
@@ -306,60 +243,10 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
             , v_partition_expression
             , v_partition_timestamp_start
             , v_partition_timestamp_end);
-        IF v_epoch = 'seconds' THEN
-            -- Non-native needs secondary, integer based constraint for epoch
-            EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%I >= %L AND %I < %L)'
-                            , v_parent_schema
-                            , v_partition_name
-                            , v_partition_name||'_partition_int_check'
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_start)::bigint
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_end)::bigint );
-        ELSIF v_epoch = 'milliseconds' THEN
-            EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%I >= %L AND %I < %L)'
-                            , v_parent_schema
-                            , v_partition_name
-                            , v_partition_name||'_partition_int_check'
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_start)::bigint * 1000
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_end)::bigint * 1000);
-        ELSIF v_epoch = 'nanoseconds' THEN
-            EXECUTE format('ALTER TABLE %I.%I ADD CONSTRAINT %I CHECK (%I >= %L AND %I < %L)'
-                            , v_parent_schema
-                            , v_partition_name
-                            , v_partition_name||'_partition_int_check'
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_start)::bigint * 1000000000
-                            , v_control
-                            , EXTRACT('epoch' from v_partition_timestamp_end)::bigint * 1000000000);
-        END IF;
+    END IF;
 
-        EXECUTE format('ALTER TABLE %I.%I INHERIT %I.%I'
-                        , v_parent_schema
-                        , v_partition_name
-                        , v_parent_schema
-                        , v_parent_tablename);
-
-        -- If custom time, set extra config options.
-        IF v_partition_type = 'time-custom' THEN
-            INSERT INTO @extschema@.custom_time_partitions (parent_table, child_table, partition_range)
-            VALUES ( p_parent_table, v_parent_schema||'.'||v_partition_name, tstzrange(v_partition_timestamp_start, v_partition_timestamp_end, '[)') );
-        END IF;
-
-        -- Indexes cannot be created on the parent, so clustering cannot be used for native yet.
-        PERFORM @extschema@.apply_cluster(v_parent_schema, v_parent_tablename, v_parent_schema, v_partition_name);
-
-        -- Foreign keys to other tables not supported in native
-        IF v_inherit_fk THEN
-            PERFORM @extschema@.apply_foreign_keys(p_parent_table, v_parent_schema||'.'||v_partition_name, v_job_id);
-        END IF;
-
-    END IF; -- end native check
-
-    -- NOTE: Privileges not automatically inherited for native. Only do so if config flag is set
-    IF v_partition_type != 'native' OR (v_partition_type = 'native' AND v_inherit_privileges = TRUE) THEN
+    -- NOTE: Privileges not automatically inherited. Only do so if config flag is set
+    IF v_inherit_privileges = TRUE THEN
         PERFORM @extschema@.apply_privileges(v_parent_schema, v_parent_tablename, v_parent_schema, v_partition_name, v_job_id);
     END IF;
 
@@ -368,36 +255,32 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
     END IF;
 
     -- Will only loop once and only if sub_partitioning is actually configured
-    -- This seemed easier than assigning a bunch of variables then doing an IF condition
-    -- This column list must be kept consistent between: 
+    -- This seemed easier than assigning a bunch of variables and doing an IF condition
+    -- This column list must be kept consistent between:
     --   create_parent, check_subpart_sameconfig, create_partition_id, create_partition_time, dump_partitioned_table_definition, and table definition
-    FOR v_row IN 
-        SELECT sub_parent
-            , sub_partition_type
+    FOR v_row IN
+        SELECT
+            sub_parent
             , sub_control
             , sub_partition_interval
-            , sub_constraint_cols
+            , sub_partition_type
             , sub_premake
-            , sub_optimize_trigger
-            , sub_optimize_constraint
-            , sub_epoch
-            , sub_inherit_fk
+            , sub_automatic_maintenance
+            , sub_template_table
             , sub_retention
             , sub_retention_schema
-            , sub_retention_keep_table
             , sub_retention_keep_index
+            , sub_retention_keep_table
+            , sub_epoch
+            , sub_constraint_cols
+            , sub_optimize_constraint
             , sub_infinite_time_partitions
-            , sub_automatic_maintenance
             , sub_jobmon
-            , sub_trigger_exception_handling
-            , sub_upsert
-            , sub_trigger_return_null
-            , sub_template_table
             , sub_inherit_privileges
             , sub_constraint_valid
-            , sub_subscription_refresh
             , sub_date_trunc_interval
             , sub_ignore_default_data
+            , sub_default_table
         FROM @extschema@.part_config_sub
         WHERE sub_parent = p_parent_table
     LOOP
@@ -407,12 +290,12 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
         v_sql := format('SELECT @extschema@.create_parent(
                  p_parent_table := %L
                 , p_control := %L
-                , p_type := %L
                 , p_interval := %L
+                , p_type := %L
+                , p_default_table := %L
                 , p_constraint_cols := %L
                 , p_premake := %L
                 , p_automatic_maintenance := %L
-                , p_inherit_fk := %L
                 , p_epoch := %L
                 , p_template_table := %L
                 , p_jobmon := %L
@@ -420,63 +303,39 @@ FOREACH v_time IN ARRAY p_partition_times LOOP
                 , p_date_trunc_interval := %L )'
             , v_parent_schema||'.'||v_partition_name
             , v_row.sub_control
-            , v_row.sub_partition_type
             , v_row.sub_partition_interval
+            , v_row.sub_partition_type
+            , v_row.sub_default_table
             , v_row.sub_constraint_cols
             , v_row.sub_premake
             , v_row.sub_automatic_maintenance
-            , v_row.sub_inherit_fk
             , v_row.sub_epoch
             , v_row.sub_template_table
             , v_row.sub_jobmon
             , p_start_partition
             , v_row.sub_date_trunc_interval);
-        
+
         RAISE DEBUG 'create_partition_time (create_parent loop): %', v_sql;
         EXECUTE v_sql;
 
-        UPDATE @extschema@.part_config SET 
+        UPDATE @extschema@.part_config SET
             retention_schema = v_row.sub_retention_schema
             , retention_keep_table = v_row.sub_retention_keep_table
-            , retention_keep_index = v_row.sub_retention_keep_index
-            , optimize_trigger = v_row.sub_optimize_trigger
             , optimize_constraint = v_row.sub_optimize_constraint
             , infinite_time_partitions = v_row.sub_infinite_time_partitions
-            , trigger_exception_handling = v_row.sub_trigger_exception_handling
-            , upsert = v_row.sub_upsert
             , inherit_privileges = v_row.sub_inherit_privileges
-            , trigger_return_null = v_row.sub_trigger_return_null
             , constraint_valid = v_row.sub_constraint_valid
-            , subscription_refresh = v_row.sub_subscription_refresh
             , ignore_default_data = v_row.sub_ignore_default_data
         WHERE parent_table = v_parent_schema||'.'||v_partition_name;
 
     END LOOP; -- end sub partitioning LOOP
 
-    -- Manage additonal constraints if set
+    -- Manage additional constraints if set
     PERFORM @extschema@.apply_constraints(p_parent_table, p_job_id := v_job_id);
 
-    IF v_publications IS NOT NULL THEN
-        -- NOTE: Native publication inheritance is only supported on PG14+
-        PERFORM @extschema@.apply_publications(p_parent_table, v_parent_schema, v_partition_name);
-    END IF;
-
     v_partition_created := true;
 
 END LOOP;
--- v_analyze is a local check if a new table is made.
--- p_analyze is a parameter to say whether to run the analyze at all. Used by create_parent() to avoid long exclusive lock or run_maintenence() to avoid long creation runs.
-IF v_analyze AND p_analyze THEN
-    IF v_jobmon_schema IS NOT NULL THEN
-        v_step_id := add_step(v_job_id, format('Analyzing partition set: %s', p_parent_table));
-    END IF;
-
-    EXECUTE format('ANALYZE %I.%I', v_parent_schema, v_parent_tablename);
-
-    IF v_jobmon_schema IS NOT NULL THEN
-        PERFORM update_step(v_step_id, 'OK', 'Done');
-    END IF;
-END IF;
 
 IF v_jobmon_schema IS NOT NULL THEN
     IF v_partition_created = false THEN
@@ -517,4 +376,3 @@ DETAIL: %
 HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
-
