@@ -1,4 +1,11 @@
-CREATE FUNCTION @extschema@.drop_partition_id(p_parent_table text, p_retention bigint DEFAULT NULL, p_keep_table boolean DEFAULT NULL, p_keep_index boolean DEFAULT NULL, p_retention_schema text DEFAULT NULL) RETURNS int
+CREATE FUNCTION @extschema@.drop_partition_id(
+    p_parent_table text
+    , p_retention bigint DEFAULT NULL
+    , p_keep_table boolean DEFAULT NULL
+    , p_keep_index boolean DEFAULT NULL
+    , p_retention_schema text DEFAULT NULL
+)
+    RETURNS int
     LANGUAGE plpgsql
     AS $$
 DECLARE
@@ -11,7 +18,6 @@ v_adv_lock                  boolean;
 v_control                   text;
 v_control_type              text;
 v_count                     int;
-v_drop_cascade_fk           boolean;
 v_drop_count                int := 0;
 v_index                     record;
 v_job_id                    bigint;
@@ -24,7 +30,6 @@ v_parent_schema             text;
 v_parent_tablename          text;
 v_partition_interval        bigint;
 v_partition_id              bigint;
-v_partition_type            text;
 v_retention                 bigint;
 v_retention_keep_index      boolean;
 v_retention_keep_table      boolean;
@@ -37,8 +42,8 @@ v_sub_parent                text;
 
 BEGIN
 /*
- * Function to drop child tables from an id-based partition set. 
- * Options to move table to different schema, drop only indexes or actually drop the table from the database.
+ * Function to drop child tables from an id-based partition set.
+ * Options to move table to different schema or actually drop the table from the database.
  */
 
 v_adv_lock := pg_try_advisory_xact_lock(hashtext('pg_partman drop_partition_id'));
@@ -48,53 +53,45 @@ IF v_adv_lock = 'false' THEN
 END IF;
 
 IF p_retention IS NULL THEN
-    SELECT  
+    SELECT
         partition_interval::bigint
-        , partition_type
         , control
         , retention::bigint
         , retention_keep_table
         , retention_keep_index
         , retention_schema
         , jobmon
-        , drop_cascade_fk
     INTO
         v_partition_interval
-        , v_partition_type
         , v_control
         , v_retention
         , v_retention_keep_table
         , v_retention_keep_index
         , v_retention_schema
         , v_jobmon
-        , v_drop_cascade_fk
-    FROM @extschema@.part_config 
-    WHERE parent_table = p_parent_table 
+    FROM @extschema@.part_config
+    WHERE parent_table = p_parent_table
     AND retention IS NOT NULL;
 
     IF v_partition_interval IS NULL THEN
         RAISE EXCEPTION 'Configuration for given parent table with a retention period not found: %', p_parent_table;
     END IF;
 ELSE -- Allow override of configuration options
-     SELECT  
+     SELECT
         partition_interval::bigint
-        , partition_type
         , control
         , retention_keep_table
         , retention_keep_index
         , retention_schema
         , jobmon
-        , drop_cascade_fk
     INTO
         v_partition_interval
-        , v_partition_type
         , v_control
         , v_retention_keep_table
         , v_retention_keep_index
         , v_retention_schema
         , v_jobmon
-        , v_drop_cascade_fk
-    FROM @extschema@.part_config 
+    FROM @extschema@.part_config
     WHERE parent_table = p_parent_table;
     v_retention := p_retention;
 
@@ -138,7 +135,7 @@ WHERE schemaname = split_part(p_parent_table, '.', 1)::name
 AND tablename = split_part(p_parent_table, '.', 2)::name;
 
 -- Loop through child tables starting from highest to get current max value in partition set
--- Avoids doing a scan on entire partition set and/or getting any values accidentally in parent.
+-- Avoids doing a scan on entire partition set and/or getting any values accidentally in default.
 FOR v_row_max_id IN
     SELECT partition_schemaname, partition_tablename FROM @extschema@.show_partitions(p_parent_table, 'DESC')
 LOOP
@@ -152,7 +149,7 @@ SELECT sub_parent INTO v_sub_parent FROM @extschema@.part_config_sub WHERE sub_p
 
 -- Loop through child tables of the given parent
 -- Must go in ascending order to avoid dropping what may be the "last" partition in the set after dropping tables that match retention period
-FOR v_row IN 
+FOR v_row IN
     SELECT partition_schemaname, partition_tablename FROM @extschema@.show_partitions(p_parent_table, 'ASC')
 LOOP
      SELECT child_start_id INTO v_partition_id FROM @extschema@.show_partition_info(v_row.partition_schemaname||'.'||v_row.partition_tablename
@@ -165,7 +162,7 @@ LOOP
         -- Do not allow final partition to be dropped if it is not a sub-partition parent
         SELECT count(*) INTO v_count FROM @extschema@.show_partitions(p_parent_table);
         IF v_count = 1 AND v_sub_parent IS NULL THEN
-            RAISE WARNING 'Attempt to drop final partition in partition set % as part of retention policy. If you see this message multiple times for the same table, advise reviewing retention policy and/or data entry into the partition set. Also consider setting "infinite_time_partitions = true" if there are large gaps in data insertion.).', p_parent_table;
+            RAISE WARNING 'Attempt to drop final partition in partition set % as part of retention policy. If you see this message multiple times for the same table, advise reviewing retention policy and/or data entry into the partition set. Also consider setting "infinite_time_partitions = true" if there are large gaps in data insertion.', p_parent_table;
             CONTINUE;
         END IF;
 
@@ -180,24 +177,47 @@ LOOP
 
         IF v_retention_keep_table = true OR v_retention_schema IS NOT NULL THEN
             -- No need to detach partition before dropping since it's going away anyway
+            -- TODO Review this to see how to handle based on recent FK issues
             -- Avoids issue of FKs not allowing detachment (Github Issue #294).
-            IF v_partition_type = 'native' THEN
-                v_sql := format('ALTER TABLE %I.%I DETACH PARTITION %I.%I'
-                    , v_parent_schema
-                    , v_parent_tablename
-                    , v_row.partition_schemaname
-                    , v_row.partition_tablename);
-                EXECUTE v_sql;
-            ELSE
-                EXECUTE format('ALTER TABLE %I.%I NO INHERIT %I.%I'
-                    , v_row.partition_schemaname
-                    , v_row.partition_tablename
-                    , v_parent_schema
-                    , v_parent_tablename);
-                IF v_jobmon_schema IS NOT NULL THEN
-                    PERFORM update_step(v_step_id, 'OK', 'Done');
-                END IF;
-            END IF;
+            v_sql := format('ALTER TABLE %I.%I DETACH PARTITION %I.%I'
+                , v_parent_schema
+                , v_parent_tablename
+                , v_row.partition_schemaname
+                , v_row.partition_tablename);
+            EXECUTE v_sql;
+
+            IF v_retention_keep_index = false THEN
+                FOR v_index IN
+                     WITH child_info AS (
+                        SELECT c1.oid
+                        FROM pg_catalog.pg_class c1
+                        JOIN pg_catalog.pg_namespace n1 ON c1.relnamespace = n1.oid
+                        WHERE c1.relname = v_row.partition_tablename::name
+                        AND n1.nspname = v_row.partition_schemaname::name
+                    )
+                    SELECT c.relname as name
+                        , con.conname
+                    FROM pg_catalog.pg_index i
+                    JOIN pg_catalog.pg_class c ON i.indexrelid = c.oid
+                    LEFT JOIN pg_catalog.pg_constraint con ON i.indexrelid = con.conindid
+                    JOIN child_info ON i.indrelid = child_info.oid
+                LOOP
+                    IF v_jobmon_schema IS NOT NULL THEN
+                        v_step_id := add_step(v_job_id, format('Drop index %s from %s.%s'
+                            , v_index.name
+                            , v_row.partition_schemaname
+                            , v_row.partition_tablename));
+                    END IF;
+                    IF v_index.conname IS NOT NULL THEN
+                        EXECUTE format('ALTER TABLE %I.%I DROP CONSTRAINT %I', v_row.partition_schemaname, v_row.partition_tablename, v_index.conname);
+                    ELSE
+                        EXECUTE format('DROP INDEX %I.%I', v_row.partition_schemaname, v_index.name);
+                    END IF;
+                    IF v_jobmon_schema IS NOT NULL THEN
+                        PERFORM update_step(v_step_id, 'OK', 'Done');
+                    END IF;
+                END LOOP;
+            END IF; -- end v_retention_keep_index IF
         END IF;
 
         IF v_retention_schema IS NULL THEN
@@ -206,49 +226,11 @@ LOOP
                     v_step_id := add_step(v_job_id, format('Drop table %s.%s', v_row.partition_schemaname, v_row.partition_tablename));
                 END IF;
                 v_sql := 'DROP TABLE %I.%I';
-                IF v_drop_cascade_fk OR v_sub_parent IS NOT NULL THEN
-                    v_sql := v_sql || ' CASCADE';
-                END IF;
                 EXECUTE format(v_sql, v_row.partition_schemaname, v_row.partition_tablename);
                 IF v_jobmon_schema IS NOT NULL THEN
                     PERFORM update_step(v_step_id, 'OK', 'Done');
                 END IF;
-            ELSIF v_retention_keep_index = false THEN
-                IF v_partition_type = 'partman' OR 
-                       ( v_partition_type = 'native' AND  current_setting('server_version_num')::int < 110000) THEN
-                    -- Cannot drop child indexes on native partition sets in PG11+
-                    FOR v_index IN 
-                         WITH child_info AS (
-                            SELECT c1.oid
-                            FROM pg_catalog.pg_class c1
-                            JOIN pg_catalog.pg_namespace n1 ON c1.relnamespace = n1.oid
-                            WHERE c1.relname = v_row.partition_tablename::name
-                            AND n1.nspname = v_row.partition_schema::name
-                        )
-                        SELECT c.relname as name
-                            , con.conname
-                        FROM pg_catalog.pg_index i
-                        JOIN pg_catalog.pg_class c ON i.indexrelid = c.oid
-                        LEFT JOIN pg_catalog.pg_constraint con ON i.indexrelid = con.conindid
-                        JOIN child_info ON i.indrelid = child_info.oid
-                    LOOP
-                        IF v_jobmon_schema IS NOT NULL THEN
-                            v_step_id := add_step(v_job_id, format('Drop index %s from %s.%s'
-                                , v_index.name
-                                , v_row.partition_schemaname
-                                , v_row.partition_tablename));
-                        END IF;
-                        IF v_index.conname IS NOT NULL THEN
-                            EXECUTE format('ALTER TABLE %I.%I DROP CONSTRAINT %I', v_row.partition_schemaname, v_row.partition_tablename, v_index.conname);
-                        ELSE
-                            EXECUTE format('DROP INDEX %I.%I', v_row.partition_schemaname, v_index.name);
-                        END IF;
-                        IF v_jobmon_schema IS NOT NULL THEN
-                            PERFORM update_step(v_step_id, 'OK', 'Done');
-                        END IF;
-                    END LOOP;
-                END IF; -- end native/11 check 
-            END IF; -- end v_retention_keep_index IF
+            END IF;
         ELSE -- Move to new schema
             IF v_jobmon_schema IS NOT NULL THEN
                 v_step_id := add_step(v_job_id, format('Moving table %s.%s to schema %s'
@@ -309,5 +291,3 @@ DETAIL: %
 HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
-
-
