@@ -1,18 +1,19 @@
 CREATE FUNCTION @extschema@.partition_data_time(
-        p_parent_table text
-        , p_batch_count int DEFAULT 1
-        , p_batch_interval interval DEFAULT NULL
-        , p_lock_wait numeric DEFAULT 0
-        , p_order text DEFAULT 'ASC'
-        , p_analyze boolean DEFAULT true
-        , p_source_table text DEFAULT NULL
-        , p_ignored_columns text[] DEFAULT NULL)
+    p_parent_table text
+    , p_batch_count int DEFAULT 1
+    , p_batch_interval interval DEFAULT NULL
+    , p_lock_wait numeric DEFAULT 0
+    , p_order text DEFAULT 'ASC'
+    , p_analyze boolean DEFAULT true
+    , p_source_table text DEFAULT NULL
+    , p_ignored_columns text[] DEFAULT NULL
+)
     RETURNS bigint
-    LANGUAGE plpgsql 
+    LANGUAGE plpgsql
     AS $$
 DECLARE
 
-v_col                       text;
+v_analyze                   boolean := FALSE;
 v_column_list               text;
 v_control                   text;
 v_control_type              text;
@@ -27,17 +28,15 @@ v_lock_iter                 int := 1;
 v_lock_obtained             boolean := FALSE;
 v_max_partition_timestamp   timestamptz;
 v_min_partition_timestamp   timestamptz;
+v_parent_schema             text;
 v_parent_tablename          text;
-v_parent_tablename_real     text;
 v_partition_expression      text;
 v_partition_interval        interval;
 v_partition_suffix          text;
 v_partition_timestamp       timestamptz[];
-v_partition_type            text;
 v_source_schemaname         text;
 v_source_tablename          text;
 v_rowcount                  bigint;
-v_sql                       text;
 v_start_control             timestamptz;
 v_total_rows                bigint := 0;
 
@@ -46,17 +45,15 @@ BEGIN
  * Populate the child table(s) of a time-based partition set with old data from the original parent
  */
 
-SELECT partition_type
-    , partition_interval::interval
+SELECT partition_interval::interval
     , control
     , datetime_string
     , epoch
-INTO v_partition_type
-    , v_partition_interval
+INTO v_partition_interval
     , v_control
     , v_datetime_string
     , v_epoch
-FROM @extschema@.part_config 
+FROM @extschema@.part_config
 WHERE parent_table = p_parent_table;
 IF NOT FOUND THEN
     RAISE EXCEPTION 'ERROR: No entry in part_config found for given table:  %', p_parent_table;
@@ -68,11 +65,12 @@ WHERE schemaname = split_part(p_parent_table, '.', 1)::name
 AND tablename = split_part(p_parent_table, '.', 2)::name;
 
 -- Preserve real parent tablename for use below
+v_parent_schema    := v_source_schemaname;
 v_parent_tablename := v_source_tablename;
 
 SELECT general_type INTO v_control_type FROM @extschema@.check_control_type(v_source_schemaname, v_source_tablename, v_control);
 
-IF v_control_type <> 'time' THEN 
+IF v_control_type <> 'time' THEN
     IF (v_control_type = 'id' AND v_epoch = 'none') OR v_control_type <> 'id' THEN
         RAISE EXCEPTION 'Cannot run on partition set without time based control column or epoch flag set with an id column. Found control: %, epoch: %', v_control_type, v_epoch;
     END IF;
@@ -94,25 +92,24 @@ IF p_source_table IS NOT NULL THEN
     END IF;
 
 
-ELSIF v_partition_type = 'native' AND current_setting('server_version_num')::int >= 110000 THEN
+ELSE
 
     IF p_batch_interval IS NOT NULL AND p_batch_interval != v_partition_interval THEN
         -- This is true because all data for a given child table must be moved out of the default partition before the child table can be created.
         -- So cannot create the child table when only some of the data has been moved out of the default partition.
-        RAISE EXCEPTION 'Custom intervals are not allowed when moving data out of the DEFAULT partition in a native set. Please leave p_interval/p_batch_interval parameters unset or NULL to allow use of partition set''s default partitioning interval.';
+        RAISE EXCEPTION 'Custom intervals are not allowed when moving data out of the DEFAULT partition. Please leave p_interval/p_batch_interval parameters unset or NULL to allow use of partition set''s default partitioning interval.';
     END IF;
-    -- Set source table to default table if PG11+, p_source_table is not set, and it exists
-    -- Otherwise just return with a DEBUG that no data source exists
-    v_sql := format('SELECT n.nspname::text, c.relname::text FROM
-        pg_catalog.pg_inherits h
-        JOIN pg_catalog.pg_class c ON c.oid = h.inhrelid
-        JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE h.inhparent = ''%I.%I''::regclass
-        AND pg_get_expr(relpartbound, c.oid) = ''DEFAULT'''
-        , v_source_schemaname
-        , v_source_tablename);
 
-    EXECUTE v_sql INTO v_default_schemaname, v_default_tablename;
+    -- Set source table to default table if p_source_table is not set, and it exists
+    -- Otherwise just return with a DEBUG that no data source exists
+    SELECT n.nspname::text, c.relname::text
+    INTO v_default_schemaname, v_default_tablename
+    FROM pg_catalog.pg_inherits h
+    JOIN pg_catalog.pg_class c ON c.oid = h.inhrelid
+    JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
+    WHERE h.inhparent = format('%I.%I', v_source_schemaname, v_source_tablename)::regclass
+    AND pg_get_expr(relpartbound, c.oid) = 'DEFAULT';
+
     IF v_default_tablename IS NOT NULL THEN
         v_source_schemaname := v_default_schemaname;
         v_source_tablename := v_default_tablename;
@@ -120,7 +117,7 @@ ELSIF v_partition_type = 'native' AND current_setting('server_version_num')::int
         v_default_exists := true;
         EXECUTE format ('CREATE TEMP TABLE IF NOT EXISTS partman_temp_data_storage (LIKE %I.%I INCLUDING INDEXES) ON COMMIT DROP', v_source_schemaname, v_source_tablename);
     ELSE
-        RAISE DEBUG 'No default table found when partition_data_id() was called';
+        RAISE DEBUG 'No default table found when partition_data_time() was called';
         RETURN v_total_rows;
     END IF;
 END IF;
@@ -139,23 +136,16 @@ v_partition_expression := CASE
 END;
 
 -- Generate column list to use in SELECT/INSERT statements below. Allows for exclusion of GENERATED (or any other desired) columns.
-v_sql := format ('SELECT ''"''||string_agg(attname, ''","'')||''"'' FROM pg_catalog.pg_attribute a
-                    JOIN pg_catalog.pg_class c ON a.attrelid = c.oid
-                    JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-                    WHERE n.nspname = %L
-                    AND c.relname = %L 
-                    AND a.attnum > 0 
-                    AND a.attisdropped = false'
-                  , v_source_schemaname
-                  , v_source_tablename);
-
-IF p_ignored_columns IS NOT NULL THEN
-    FOREACH v_col IN ARRAY p_ignored_columns LOOP
-        v_sql := v_sql || format(' AND attname != %L ', v_col);
-    END LOOP;
-END IF;
-
-EXECUTE v_sql INTO v_column_list;
+SELECT string_agg(quote_ident(attname), ',')
+INTO v_column_list
+FROM pg_catalog.pg_attribute a
+JOIN pg_catalog.pg_class c ON a.attrelid = c.oid
+JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
+WHERE n.nspname = v_source_schemaname
+AND c.relname = v_source_tablename
+AND a.attnum > 0
+AND a.attisdropped = false
+AND attname <> ALL(COALESCE(p_ignored_columns, ARRAY[]::text[]));
 
 FOR i IN 1..p_batch_count LOOP
 
@@ -171,55 +161,31 @@ FOR i IN 1..p_batch_count LOOP
         EXIT;
     END IF;
 
-    IF v_partition_type = 'partman' THEN
-        CASE
-            WHEN v_partition_interval = '15 mins' THEN
-                v_min_partition_timestamp := date_trunc('hour', v_start_control) + 
-                    '15min'::interval * floor(date_part('minute', v_start_control) / 15.0);
-            WHEN v_partition_interval = '30 mins' THEN
-                v_min_partition_timestamp := date_trunc('hour', v_start_control) + 
-                    '30min'::interval * floor(date_part('minute', v_start_control) / 30.0);
-            WHEN v_partition_interval = '1 hour' THEN
-                v_min_partition_timestamp := date_trunc('hour', v_start_control);
-            WHEN v_partition_interval = '1 day' THEN
-                v_min_partition_timestamp := date_trunc('day', v_start_control);
-            WHEN v_partition_interval = '1 week' THEN
-                v_min_partition_timestamp := date_trunc('week', v_start_control);
-            WHEN v_partition_interval = '1 month' THEN
-                v_min_partition_timestamp := date_trunc('month', v_start_control);
-            WHEN v_partition_interval = '3 months' THEN
-                v_min_partition_timestamp := date_trunc('quarter', v_start_control);
-            WHEN v_partition_interval = '1 year' THEN
-                v_min_partition_timestamp := date_trunc('year', v_start_control);
-        END CASE;
-    ELSIF v_partition_type IN ('time-custom', 'native') THEN
-        SELECT child_start_time INTO v_min_partition_timestamp FROM @extschema@.show_partition_info(v_source_schemaname||'.'||v_last_partition
-            , v_partition_interval::text
-            , p_parent_table);
-        v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
-        LOOP
-            IF v_start_control >= v_min_partition_timestamp AND v_start_control < v_max_partition_timestamp THEN
-                EXIT;
-            ELSE
-                BEGIN
-                    IF v_start_control >= v_max_partition_timestamp THEN
-                        -- Keep going forward in time, checking if child partition time interval encompasses the current v_start_control value
-                        v_min_partition_timestamp := v_max_partition_timestamp;
-                        v_max_partition_timestamp := v_max_partition_timestamp + v_partition_interval;
+    SELECT child_start_time INTO v_min_partition_timestamp FROM @extschema@.show_partition_info(v_source_schemaname||'.'||v_last_partition
+        , v_partition_interval::text
+        , p_parent_table);
+    v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
+    LOOP
+        IF v_start_control >= v_min_partition_timestamp AND v_start_control < v_max_partition_timestamp THEN
+            EXIT;
+        ELSE
+            BEGIN
+                IF v_start_control >= v_max_partition_timestamp THEN
+                    -- Keep going forward in time, checking if child partition time interval encompasses the current v_start_control value
+                    v_min_partition_timestamp := v_max_partition_timestamp;
+                    v_max_partition_timestamp := v_max_partition_timestamp + v_partition_interval;
 
-                    ELSE
-                        -- Keep going backwards in time, checking if child partition time interval encompasses the current v_start_control value
-                        v_max_partition_timestamp := v_min_partition_timestamp;
-                        v_min_partition_timestamp := v_min_partition_timestamp - v_partition_interval;
-                    END IF;
-                EXCEPTION WHEN datetime_field_overflow THEN
-                    RAISE EXCEPTION 'Attempted partition time interval is outside PostgreSQL''s supported time range. 
-                        Unable to create partition with interval before timestamp % ', v_min_partition_timestamp;
-                END;
-            END IF;
-        END LOOP;
-
-    END IF;
+                ELSE
+                    -- Keep going backwards in time, checking if child partition time interval encompasses the current v_start_control value
+                    v_max_partition_timestamp := v_min_partition_timestamp;
+                    v_min_partition_timestamp := v_min_partition_timestamp - v_partition_interval;
+                END IF;
+            EXCEPTION WHEN datetime_field_overflow THEN
+                RAISE EXCEPTION 'Attempted partition time interval is outside PostgreSQL''s supported time range.
+                    Unable to create partition with interval before timestamp % ', v_min_partition_timestamp;
+            END;
+        END IF;
+    END LOOP;
 
     v_partition_timestamp := ARRAY[v_min_partition_timestamp];
     IF p_order = 'ASC' THEN
@@ -271,7 +237,7 @@ FOR i IN 1..p_batch_count LOOP
     v_current_partition_name := @extschema@.check_name_length(v_parent_tablename, v_partition_suffix, TRUE);
 
     IF v_default_exists THEN
-        -- Child tables cannot be created in native partitioning if data that belongs to it exists in the default
+        -- Child tables cannot be created if data that belongs to it exists in the default
         -- Have to move data out to temporary location, create child table, then move it back
 
         -- Temp table created above to avoid excessive temp creation in loop
@@ -285,7 +251,8 @@ FOR i IN 1..p_batch_count LOOP
             , v_max_partition_timestamp
             , v_column_list);
 
-        PERFORM @extschema@.create_partition_time(p_parent_table, v_partition_timestamp, p_analyze);
+        -- Set analyze to true if a table is created
+        v_analyze := @extschema@.create_partition_time(p_parent_table, v_partition_timestamp);
 
         EXECUTE format('WITH partition_data AS (
                 DELETE FROM partman_temp_data_storage RETURNING *)
@@ -296,7 +263,8 @@ FOR i IN 1..p_batch_count LOOP
 
     ELSE
 
-        PERFORM @extschema@.create_partition_time(p_parent_table, v_partition_timestamp, p_analyze);
+        -- Set analyze to true if a table is created
+        v_analyze := @extschema@.create_partition_time(p_parent_table, v_partition_timestamp);
 
         EXECUTE format('WITH partition_data AS (
                             DELETE FROM ONLY %I.%I WHERE %s >= %L AND %3$s < %5$L RETURNING *)
@@ -316,14 +284,17 @@ FOR i IN 1..p_batch_count LOOP
         EXIT;
     END IF;
 
-END LOOP; 
+END LOOP;
 
-IF v_partition_type IN ('partman', 'time-custom') THEN
-    PERFORM @extschema@.create_function_time(p_parent_table);
+-- v_analyze is a local check if a new table is made.
+-- p_analyze is a parameter to say whether to run the analyze at all. Used by create_parent() to avoid long exclusive lock or run_maintenence() to avoid long creation runs.
+IF v_analyze AND p_analyze THEN
+    RAISE DEBUG 'partiton_data_time: Begin analyze of %.%', v_parent_schema, v_parent_tablename;
+    EXECUTE format('ANALYZE %I.%I', v_parent_schema, v_parent_tablename);
+    RAISE DEBUG 'partiton_data_time: End analyze of %.%', v_parent_schema, v_parent_tablename;
 END IF;
 
 RETURN v_total_rows;
 
 END
 $$;
-
