@@ -1,4 +1,9 @@
-CREATE FUNCTION @extschema@.show_partitions (p_parent_table text, p_order text DEFAULT 'ASC', p_include_default boolean DEFAULT false) RETURNS TABLE (partition_schemaname text, partition_tablename text)
+CREATE FUNCTION @extschema@.show_partitions (
+    p_parent_table text
+    , p_order text DEFAULT 'ASC'
+    , p_include_default boolean DEFAULT false
+)
+    RETURNS TABLE (partition_schemaname text, partition_tablename text)
     LANGUAGE plpgsql STABLE
     SET search_path = @extschema@,pg_temp
     AS $$
@@ -9,9 +14,9 @@ v_control_type          text;
 v_datetime_string       text;
 v_default_sql           text;
 v_epoch                 text;
+v_epoch_divisor         bigint;
 v_parent_schema         text;
 v_parent_tablename      text;
-v_partition_interval    text;
 v_partition_type        text;
 v_sql                   text;
 
@@ -23,17 +28,14 @@ BEGIN
  */
 
 IF upper(p_order) NOT IN ('ASC', 'DESC') THEN
-    EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
-    RAISE EXCEPTION 'p_order paramter must be one of the following values: ASC, DESC';
+    RAISE EXCEPTION 'p_order parameter must be one of the following values: ASC, DESC';
 END IF;
 
 SELECT partition_type
-    , partition_interval
     , datetime_string
     , control
     , epoch
 INTO v_partition_type
-    , v_partition_interval
     , v_datetime_string
     , v_control
     , v_epoch
@@ -58,55 +60,61 @@ SELECT general_type INTO v_control_type FROM @extschema@.check_control_type(v_pa
 
 RAISE DEBUG 'show_partitions: v_parent_schema: %, v_parent_tablename: %, v_datetime_string: %', v_parent_schema, v_parent_tablename, v_datetime_string;
 
-v_sql := format('SELECT n.nspname::text AS partition_schemaname, c.relname::text AS partition_name FROM
-        pg_catalog.pg_inherits h
+v_sql := format('SELECT n.nspname::text AS partition_schemaname
+        , c.relname::text AS partition_name
+        FROM pg_catalog.pg_inherits h
         JOIN pg_catalog.pg_class c ON c.oid = h.inhrelid
         JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
         WHERE h.inhparent = ''%I.%I''::regclass'
     , v_parent_schema
     , v_parent_tablename);
 
-IF v_partition_type = 'native' AND current_setting('server_version_num')::int >= 110000 THEN
-
-    IF p_include_default THEN
-        -- Return the default partition immediately as first item in list
-        v_default_sql := v_sql || format('
-            AND pg_get_expr(relpartbound, c.oid) = ''DEFAULT''');
-        RAISE DEBUG 'show_partitions: v_default_sql: %', v_default_sql;
-        RETURN QUERY EXECUTE v_default_sql;
-    END IF;
-
-    v_sql := v_sql || format('
-        AND pg_get_expr(relpartbound, c.oid) != ''DEFAULT''');
 
+IF p_include_default THEN
+    -- Return the default partition immediately as first item in list
+    v_default_sql := v_sql || format('
+        AND pg_get_expr(relpartbound, c.oid) = ''DEFAULT''');
+    RAISE DEBUG 'show_partitions: v_default_sql: %', v_default_sql;
+    RETURN QUERY EXECUTE v_default_sql;
 END IF;
 
-IF v_control_type = 'time' OR (v_control_type = 'id' AND v_epoch <> 'none') THEN
+v_sql := v_sql || format('
+    AND pg_get_expr(relpartbound, c.oid) != ''DEFAULT'' ');
 
-    IF v_partition_interval::interval <> '3 months' OR (v_partition_interval::interval = '3 months' AND v_partition_type = 'time-custom') THEN
-        v_sql := v_sql || format('
-            ORDER BY to_timestamp(substring(c.relname from ((length(c.relname) - position(''p_'' in reverse(c.relname))) + 2) ), %L) %s'
-            , v_datetime_string
-            , p_order);
 
-    ELSE
+IF v_control_type = 'time' THEN
 
-        -- For quarterly, to_timestamp() doesn't recognize "Q" in datetime string. 
-        -- First order by just the year, then order by the quarter number (should be last character in table name)
-        v_sql := v_sql || format('
-            ORDER BY to_timestamp(substring(c.relname from ((length(c.relname) - position(''p_'' in reverse(c.relname))) + 2) for 4), ''YYYY'') %s
-            , substring(reverse(c.relname) from 1 for 1) %s'
-            , p_order
-            , p_order);
+    v_sql := v_sql || format('
+        ORDER BY (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(([^)]+)\) TO \(([^)]+)\)$REGEX$))[1]::text::timestamptz %s '
+        , p_order);
 
+ELSIF v_control_type = 'id' AND v_epoch <> 'none' THEN
+
+    IF v_epoch = 'seconds' THEN
+        v_epoch_divisor := 1;
+    ELSIF v_epoch = 'milliseconds' THEN
+        v_epoch_divisor := 1000;
+    ELSIF v_epoch = 'nanoseconds' THEN
+        v_epoch_divisor := 1000000000;
     END IF;
 
+    -- Have to do a trim here because of inconsistency in quoting different integer types. Ex: bigint boundary values are quoted but int values are not
+    v_sql := v_sql || format('
+        ORDER BY to_timestamp(trim( BOTH $QUOTE$''$QUOTE$ from (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(([^)]+)\) TO \(([^)]+)\)$REGEX$))[1]::text )::bigint /%s ) %s '
+        , v_epoch_divisor
+        , p_order);
+
 ELSIF v_control_type = 'id' THEN
 
+    -- Have to do a trim here because of inconsistency in quoting different integer types. Ex: bigint boundary values are quoted but int values are not
     v_sql := v_sql || format('
-        ORDER BY substring(c.relname from ((length(c.relname) - position(''p_'' in reverse(c.relname))) + 2) )::bigint %s'
+        ORDER BY trim( BOTH $QUOTE$''$QUOTE$ from (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(([^)]+)\) TO \(([^)]+)\)$REGEX$))[1]::text )::bigint %s '
         , p_order);
 
+ELSE
+
+    RAISE EXCEPTION 'show_partitions: Unexpected code path in sort order determination. Please report the steps that lead to this error to extension maintainers.';
+
 END IF;
 
 RAISE DEBUG 'show_partitions: v_sql: %', v_sql;
@@ -115,4 +123,3 @@ RETURN QUERY EXECUTE v_sql;
 
 END
 $$;
-
